@@ -24,6 +24,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <qbuttongroup.h>
 #include <qcheckbox.h>
 #include <qlabel.h>
@@ -48,6 +50,8 @@
 #include <kstringhandler.h>
 #include <kurlrequester.h>
 #include <kwin.h>
+#include <kimagefilepreview.h>
+#include <knewstuff/downloaddialog.h>
 
 #include "bgmonitor.h"
 #include "bgwallpaper.h"
@@ -76,15 +80,17 @@ BGDialog::BGDialog(QWidget* parent, KConfig* _config, bool _multidesktop)
    }
 
    // preview monitor
-   m_monitorImage->setPixmap( UserIcon( "monitor" ));
+   m_monitorImage->setPixmap( QPixmap( locate("data",  "kcontrol/pics/monitor.png") ) );
    m_monitorImage->setFixedSize(m_monitorImage->sizeHint());
    m_pMonitor = new BGMonitor(m_monitorImage, "preview monitor");
    m_pMonitor->setGeometry(23, 14, 151, 115);
    connect(m_pMonitor, SIGNAL(imageDropped(const QString &)), SLOT(slotImageDropped(const QString &)));
-
-   // desktop
-   connect(m_comboDesktop, SIGNAL(activated(int)),
-           SLOT(slotSelectDesk(int)));
+   if( m_multidesktop)
+     {
+       // desktop
+       connect(m_comboDesktop, SIGNAL(activated(int)),
+	       SLOT(slotSelectDesk(int)));
+     }
 
    // background image settings
    QIconSet iconSet = SmallIconSet(QString::fromLatin1("fileopen"));
@@ -123,6 +129,9 @@ BGDialog::BGDialog(QWidget* parent, KConfig* _config, bool _multidesktop)
    connect(m_buttonAdvanced, SIGNAL(clicked()),
            SLOT(slotAdvanced()));
 
+   connect(m_buttonGetNew, SIGNAL(clicked()),
+           SLOT(slotGetNewStuff()));
+
    // renderers
    m_Renderer = QPtrVector<KBackgroundRenderer>( m_Max + 1 );
    m_Renderer.setAutoDelete(true);
@@ -187,6 +196,8 @@ void BGDialog::makeReadOnly()
     m_comboBlend->setEnabled( false );
     m_sliderBlend->setEnabled( false );
     m_cbBlendReverse->setEnabled( false );
+    m_buttonAdvanced->setEnabled( false );
+    m_buttonGetNew->setEnabled( false );
 }
 
 void BGDialog::load()
@@ -319,60 +330,7 @@ void BGDialog::initUI()
          m_comboPattern->insertItem(pat.comment());
    }
 
-   // Wallpapers
-   // the following QMap is lower cased names mapped to cased names and URLs
-   // this way we get case insensitive sorting
-   QMap<QString, QPair<QString, QString> > papers;
-   QStringList lst = m_pDirs->findAllResources("wallpaper", "*", false, true);
-   for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
-   {
-      // First try to see if we have a comment describing the image.  If we do
-      // just use the first line of said comment.
-
-      KFileMetaInfo metaInfo(*it);
-      QString imageCaption;
-
-      if (metaInfo.isValid() && metaInfo.item("Comment").isValid())
-         imageCaption = metaInfo.item("Comment").string().section('\n', 0, 0);
-
-      if (imageCaption.isEmpty())
-      {
-         int slash = (*it).findRev('/') + 1;
-         int endDot = (*it).findRev('.');
-
-         // strip the extension if it exists
-         if (endDot != -1 && endDot > slash)
-            imageCaption = (*it).mid(slash, endDot - slash);
-         else
-            imageCaption = (*it).mid(slash);
-
-         imageCaption.replace('_', ' ');
-         imageCaption = KStringHandler::capwords(imageCaption);
-      }
-      
-      imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
-
-      // avoid name collisions
-      QString rs = imageCaption;
-      QString lrs = rs.lower();
-      for (int n = 1; papers.find(lrs) != papers.end(); ++n)
-      {
-         rs = imageCaption + " (" + QString::number(n) + ')';
-         lrs = rs.lower();
-      }
-      papers[lrs] = qMakePair(rs, *it);
-   }
-
-   KComboBox *comboWallpaper = m_urlWallpaperBox;
-   int i = 0;
-   for (QMap<QString, QPair<QString, QString> >::Iterator it = papers.begin();
-        it != papers.end();
-        ++it)
-   {
-      comboWallpaper->insertItem(it.data().first);
-      m_Wallpaper[it.data().second] = i;
-      i++;
-   }
+   loadWallpaperFilesList();
 
    // Wallpaper tilings: again they must match the ones from bgrender.cc
    m_comboWallpaperPos->insertItem(i18n("Centered"));
@@ -382,6 +340,7 @@ void BGDialog::initUI()
    m_comboWallpaperPos->insertItem(i18n("Tiled Maxpect"));
    m_comboWallpaperPos->insertItem(i18n("Scaled"));
    m_comboWallpaperPos->insertItem(i18n("Centered Auto Fit"));
+   m_comboWallpaperPos->insertItem(i18n("Scale & Crop"));
 
    // Blend modes: make sure these match with kdesktop/bgrender.cc !!
    m_comboBlend->insertItem(i18n("No Blending"));
@@ -397,6 +356,109 @@ void BGDialog::initUI()
    m_comboBlend->insertItem(i18n("Hue Shift"));
 }
 
+void BGDialog::loadWallpaperFilesList() {
+
+   // Wallpapers
+   // the following QMap is lower cased names mapped to cased names and URLs
+   // this way we get case insensitive sorting
+   QMap<QString, QPair<QString, QString> > papers;
+
+   //search for .desktop files before searching for images without .desktop files
+   QStringList lst = m_pDirs->findAllResources("wallpaper", "*desktop", false, true);
+   QStringList files;
+   for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
+   {
+      KSimpleConfig fileConfig(*it);
+      fileConfig.setGroup("Wallpaper");
+
+      QString imageCaption = fileConfig.readEntry("Name");
+      QString fileName = fileConfig.readEntry("File");
+
+      if (imageCaption.isEmpty())
+      {
+         imageCaption = fileName;
+         imageCaption.replace('_', ' ');
+         imageCaption = KStringHandler::capwords(imageCaption);
+      }
+
+      imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
+
+      // avoid name collisions
+      QString rs = imageCaption;
+      QString lrs = rs.lower();
+      for (int n = 1; papers.find(lrs) != papers.end(); ++n)
+      {
+         rs = imageCaption + " (" + QString::number(n) + ')';
+         lrs = rs.lower();
+      }
+      int slash = (*it).findRev('/') + 1;
+      QString directory = (*it).left(slash);
+      bool canLoadScaleable = false;
+#ifdef HAVE_LIBART
+      canLoadScaleable = true;
+#endif
+      if ( fileConfig.readEntry("ImageType") == "pixmap" || canLoadScaleable ) {
+	      papers[lrs] = qMakePair(rs, directory + fileName);
+	      files.append(directory + fileName);
+      }
+   }
+
+   //now find any wallpapers that don't have a .desktop file
+   lst = m_pDirs->findAllResources("wallpaper", "*", false, true);
+   for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
+   {
+      if ( !(*it).endsWith(".desktop") && files.grep(*it).empty() ) {
+         // First try to see if we have a comment describing the image.  If we do
+         // just use the first line of said comment.
+         KFileMetaInfo metaInfo(*it);
+         QString imageCaption;
+
+         if (metaInfo.isValid() && metaInfo.item("Comment").isValid())
+            imageCaption = metaInfo.item("Comment").string().section('\n', 0, 0);
+
+         if (imageCaption.isEmpty())
+         {
+            int slash = (*it).findRev('/') + 1;
+            int endDot = (*it).findRev('.');
+
+            // strip the extension if it exists
+            if (endDot != -1 && endDot > slash)
+               imageCaption = (*it).mid(slash, endDot - slash);
+            else
+               imageCaption = (*it).mid(slash);
+
+            imageCaption.replace('_', ' ');
+            imageCaption = KStringHandler::capwords(imageCaption);
+         }
+
+         imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
+
+         // avoid name collisions
+         QString rs = imageCaption;
+         QString lrs = rs.lower();
+         for (int n = 1; papers.find(lrs) != papers.end(); ++n)
+         {
+            rs = imageCaption + " (" + QString::number(n) + ')';
+            lrs = rs.lower();
+         }
+         papers[lrs] = qMakePair(rs, *it);
+      }
+   }
+
+   KComboBox *comboWallpaper = m_urlWallpaperBox;
+   comboWallpaper->clear();
+   m_Wallpaper.clear();
+   int i = 0;
+   for (QMap<QString, QPair<QString, QString> >::Iterator it = papers.begin();
+        it != papers.end();
+        ++it)
+   {
+      comboWallpaper->insertItem(it.data().first);
+      m_Wallpaper[it.data().second] = i;
+      i++;
+   }
+}
+
 void BGDialog::setWallpaper(const QString &s)
 {
    KComboBox *comboWallpaper = m_urlWallpaperBox;
@@ -405,12 +467,21 @@ void BGDialog::setWallpaper(const QString &s)
    if (m_Wallpaper.find(s) == m_Wallpaper.end())
    {
       int i = comboWallpaper->count();
-      if (comboWallpaper->text(i-1) == s)
+      QString imageCaption;
+      int slash = s.findRev('/') + 1;
+      int endDot = s.findRev('.');
+
+      // strip the extension if it exists
+      if (endDot != -1 && endDot > slash)
+         imageCaption = s.mid(slash, endDot - slash);
+      else
+         imageCaption = s.mid(slash);
+      if (comboWallpaper->text(i-1) == imageCaption)
       {
          i--;
          comboWallpaper->removeItem(i);
       }
-      comboWallpaper->insertItem(KStringHandler::lsqueeze(s, 45));
+      comboWallpaper->insertItem(KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11));
       m_Wallpaper[s] = i;
       comboWallpaper->setCurrentItem(i);
    }
@@ -426,7 +497,15 @@ void BGDialog::slotWallpaperSelection()
    KFileDialog dlg( QString::null, QString::null, this,
                     "file dialog", true );
 
-   dlg.setFilter(KImageIO::pattern());
+   KImageFilePreview* previewWidget = new KImageFilePreview(&dlg);
+   dlg.setPreviewWidget(previewWidget);
+
+   QString pattern = KImageIO::pattern();
+#ifdef HAVE_LIBART
+   pattern = "*.svg *.svgz " + pattern;
+   pattern += i18n("\n*.svg *.SVG *.svgz *.SVGZ|Scalable Vector Graphics");
+#endif
+   dlg.setFilter(pattern);
    dlg.setMode( KFile::File | KFile::ExistingOnly | KFile::LocalOnly );
    dlg.setCaption( i18n("Select Wallpaper") );
 
@@ -467,18 +546,19 @@ void BGDialog::updateUI()
    m_colorPrimary->setColor(r->colorA());
    m_colorSecondary->setColor(r->colorB());
 
-   if (r->backgroundMode() == KBackgroundSettings::Program)
+   int wallpaperMode = r->wallpaperMode();
+   int multiMode = r->multiWallpaperMode();
+
+   if (r->backgroundMode() == KBackgroundSettings::Program &&
+       wallpaperMode == KBackgroundSettings::NoWallpaper)
       groupBox3->setEnabled( false );
    else
       groupBox3->setEnabled( true );
 
-   int wallpaperMode = r->wallpaperMode();
-   int multiMode = r->multiWallpaperMode();
-
    if ((multiMode == KBackgroundSettings::NoMultiRandom) ||
        (multiMode == KBackgroundSettings::NoMulti))
    {
-      // No wallper
+      // No wallpaper
       if (wallpaperMode == KBackgroundSettings::NoWallpaper )
       {
          m_urlWallpaperBox->setEnabled(false);
@@ -487,7 +567,7 @@ void BGDialog::updateUI()
          m_comboWallpaperPos->setEnabled(false);
          m_lblWallpaperPos->setEnabled(false);
          m_buttonGroupBackground->setButton(
-            m_buttonGroupBackground->id(m_radioNoPicture) );
+         m_buttonGroupBackground->id(m_radioNoPicture) );
       }
 
       // 1 Picture
@@ -500,7 +580,7 @@ void BGDialog::updateUI()
          m_lblWallpaperPos->setEnabled(true);
          setWallpaper(r->wallpaper());
          m_buttonGroupBackground->setButton(
-            m_buttonGroupBackground->id(m_radioPicture) );
+         m_buttonGroupBackground->id(m_radioPicture) );
       }
    }
 
@@ -513,7 +593,7 @@ void BGDialog::updateUI()
       m_comboWallpaperPos->setEnabled(true);
       m_lblWallpaperPos->setEnabled(true);
       m_buttonGroupBackground->setButton(
-         m_buttonGroupBackground->id(m_radioSlideShow) );
+      m_buttonGroupBackground->id(m_radioSlideShow) );
    }
 
    m_comboWallpaperPos->setCurrentItem(r->wallpaperMode()-1);
@@ -866,6 +946,8 @@ void BGDialog::slotAdvanced()
     dlg.setTextColor(m_pGlobals->textColor());
     dlg.setTextBackgroundColor(m_pGlobals->textBackgroundColor());
     dlg.setShadowEnabled(m_pGlobals->shadowEnabled());
+    dlg.setTextLines(m_pGlobals->textLines());
+    dlg.setTextWidth(m_pGlobals->textWidth());
 
     if (m_pGlobals->limitCache())
        dlg.setCacheSize( m_pGlobals->cacheSize() );
@@ -891,6 +973,8 @@ void BGDialog::slotAdvanced()
     m_pGlobals->setTextColor(dlg.textColor());
     m_pGlobals->setTextBackgroundColor(dlg.textBackgroundColor());
     m_pGlobals->setShadowEnabled(dlg.shadowEnabled());
+    m_pGlobals->setTextLines(dlg.textLines());
+    m_pGlobals->setTextWidth(dlg.textWidth());
 
     r->stop();
     m_previewUpdates = true;
@@ -901,6 +985,21 @@ void BGDialog::slotAdvanced()
     emit changed(true);
 }
 
+void BGDialog::slotGetNewStuff()
+{
+   //FIXME set this to a server when we get one
+   //should really be in a .rc file but could be either
+   //kcmshellrc or kcontrolrc
+   KConfig* config = KGlobal::config();
+   config->setGroup("KNewStuff");
+   config->writeEntry( "ProvidersUrl", "http://download.kde.org/khotnewstuff/wallpaper-providers.xml" );
+   config->writeEntry( "StandardResource", "wallpaper" );
+   config->sync();
+
+   KNS::DownloadDialog::open("wallpapers");
+   loadWallpaperFilesList();
+}
+
 void BGDialog::slotBlendMode(int mode)
 {
    if (mode == m_Renderer[m_eDesk]->blendMode())
