@@ -17,7 +17,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301, USA.             *
  ***************************************************************************/
 
 #if defined(INFO_OPENGL_AVAILABLE)
@@ -109,10 +109,15 @@ static bool get_dri_device()
     if (!line.isEmpty()) {
 	dri_info.module = line.mid(0, line.find(0x20));
 
-	QRegExp rx = QRegExp("\\bPCI:([0-9a-fA-F]:[0-9a-fA-F]:[0-9a-fA-F])\\b");
+	// possible formats, for regression testing
+	// line = " PCI:01:00:0";
+	// line = " pci:0000:01:00.0"
+	QRegExp rx = QRegExp("\\b[Pp][Cc][Ii][:]([0-9a-fA-F]+[:])?([0-9a-fA-F]+[:][0-9a-fA-F]+[:.][0-9a-fA-F]+)\\b");
 	if (rx.search(line)>0)	 {
-		dri_info.pci = rx.cap(1);
+		dri_info.pci = rx.cap(2);
 		int end = dri_info.pci.findRev(':');
+		int end2 = dri_info.pci.findRev('.');
+		if (end2>end) end=end2;
 		dri_info.pci[end]='.';
 
 		QString cmd = QString("lspci -m -v -s ") + dri_info.pci;
@@ -230,6 +235,9 @@ print_limits(QListViewItem *l1, const char * glExtensions, bool GetProcAddress)
       GL_COMPRESSED_TEXTURE_FORMATS
 */
 
+  if (!glExtensions)
+	return;
+
   struct token_name {
       GLuint type;  // count and flags, !!! count must be <=2 for now
       GLenum token;
@@ -424,12 +432,12 @@ print_limits(QListViewItem *l1, const char * glExtensions, bool GetProcAddress)
 	for (cur_token = groups[i].group; cur_token->type; cur_token++) {
 
    		bool tfloat = cur_token->type & KCMGL_FLOAT;
-   		bool tprog = cur_token->type & KCMGL_PROG;
 		int count = KCMGL_COUNT_MASK(cur_token->type);
 		GLint max[2]={0,0};
    		GLfloat fmax[2]={0.0,0.0};
 
 #if defined(PFNGLGETPROGRAMIVARBPROC) && defined(GL_ARB_vertex_program)
+   		bool tprog = cur_token->type & KCMGL_PROG;
 		if (tprog && kcm_glGetProgramivARB)
 			kcm_glGetProgramivARB(groups[i].type, cur_token->token, max);
 		else
