@@ -50,18 +50,26 @@ KDMUsersWidget::KDMUsersWidget(QWidget *parent, const char *name)
 {
 #ifdef __linux__
     struct stat st;
-    if (!stat( "/etc/debian_version", &st ) || !stat( "/usr/portage", &st ))
-	defminuid = "1000";	// debian, gentoo
-    else if (!stat( "/etc/mandrake-release", &st ))	// check before redhat!
-	defminuid = "500";	// mandrake
-    else if (!stat( "/etc/redhat-release", &st ))
-	defminuid = "100";	// redhat
-    else //if (!stat( "/etc/SuSE-release", &st ))
-	defminuid = "500";	// suse
+    if (!stat( "/etc/debian_version", &st )) {	/* debian */
+	defminuid = "1000";
+	defmaxuid = "29999";
+    } else if (!stat( "/usr/portage", &st )) {	/* gentoo */
+	defminuid = "1000";
+	defmaxuid = "65000";
+    } else if (!stat( "/etc/mandrake-release", &st )) {	/* mandrake - check before redhat! */
+	defminuid = "500";
+	defmaxuid = "65000";
+    } else if (!stat( "/etc/redhat-release", &st )) {	/* redhat */
+	defminuid = "100";
+	defmaxuid = "65000";
+    } else /* if (!stat( "/etc/SuSE-release", &st )) */ {	/* suse */
+	defminuid = "500";
+	defmaxuid = "65000";
+    }
 #else
     defminuid = "1000";
-#endif
     defmaxuid = "65000";
+#endif
 
     // We assume that $kde_datadir/kdm exists, but better check for pics/ and pics/users,
     // and create them if necessary.
@@ -71,6 +79,7 @@ KDMUsersWidget::KDMUsersWidget(QWidget *parent, const char *name)
     QDir testDir( m_userPixDir );
     if ( !testDir.exists() && !testDir.mkdir( testDir.absPath() ) && !geteuid() )
         KMessageBox::sorry( this, i18n("Unable to create folder %1").arg( testDir.absPath() ) );
+    chmod( QFile::encodeName( m_userPixDir ), 0755 );
 
     m_defaultText = i18n("<default>");
 
@@ -97,30 +106,31 @@ KDMUsersWidget::KDMUsersWidget(QWidget *parent, const char *name)
     connect(lemaxuid, SIGNAL(textChanged( const QString & )), SLOT(slotChanged()) );
     connect(lemaxuid, SIGNAL(textChanged( const QString & )), SLOT(slotMinMaxChanged()) );
 
-    usrGroup = new QButtonGroup( 5, Qt::Vertical, i18n("Show Users"), this );
+    usrGroup = new QButtonGroup( 5, Qt::Vertical, i18n("Users"), this );
     connect( usrGroup, SIGNAL(clicked( int )), SLOT(slotShowOpts()) );
     connect( usrGroup, SIGNAL(clicked( int )), SLOT(slotChanged()) );
-    rbnoneusr = new QRadioButton( i18n("&None"), usrGroup );
-    QWhatsThis::add( rbnoneusr, i18n("If this option is selected, KDM will not show any users."
-      " If one of the alternative radio buttons is selected, KDM will show a list of users,"
+    cbshowlist = new QCheckBox( i18n("Show list"), usrGroup );
+    QWhatsThis::add( cbshowlist, i18n("If this option is checked, KDM will show a list of users,"
       " so users can click on their name or image rather than typing in their login.") );
-    rbselusr = new QRadioButton( i18n("selected users", "Selected onl&y"), usrGroup );
-    QWhatsThis::add( rbselusr, i18n("If this option is selected, KDM will only show the users"
-      " checked in the \"Select users\" list.") );
-    rballusr = new QRadioButton( i18n("not hidden users", "Not hidden"), usrGroup );
-    QWhatsThis::add( rballusr, i18n("If this option is selected, KDM will show all non-system users, except those"
-      " checked in the \"Select users\" list."));
+    cbcomplete = new QCheckBox( i18n("Autocompletion"), usrGroup );
+    QWhatsThis::add( cbcomplete, i18n("If this option is checked, KDM will automatically complete"
+      " user names while they are typed in the line edit.") );
+    cbinverted = new QCheckBox( i18n("Inverse selection"), usrGroup );
+    QWhatsThis::add( cbinverted, i18n("This option specifies how the users for \"Show list\" and \"Autocompletion\""
+      " are selected in the \"Select users and groups\" list: "
+      "If not checked, select only the checked users. "
+      "If checked, select all non-system users, except the checked ones."));
     cbusrsrt = new QCheckBox( i18n("Sor&t users"), usrGroup );
     connect( cbusrsrt, SIGNAL(toggled( bool )), SLOT(slotChanged()) );
     QWhatsThis::add( cbusrsrt, i18n("If this is checked, KDM will alphabetically sort the user list."
       " Otherwise users are listed in the order they appear in the password file.") );
 
     wstack = new QWidgetStack( this );
-    s_label = new QLabel( wstack, i18n("S&elect users:"), this );
+    s_label = new QLabel( wstack, i18n("S&elect users and groups:"), this );
     optinlv = new KListView( this );
     optinlv->addColumn( i18n("Selected Users") );
     optinlv->setResizeMode( QListView::LastColumn );
-    QWhatsThis::add( optinlv, i18n("KDM will show all checked users.") );
+    QWhatsThis::add( optinlv, i18n("KDM will show all checked users. Entries denoted with '@' are user groups. Checking a group is like checking all users in that group.") );
     wstack->addWidget( optinlv );
     connect( optinlv, SIGNAL(clicked( QListViewItem * )),
 	     SLOT(slotUpdateOptIn( QListViewItem * )) );
@@ -129,7 +139,7 @@ KDMUsersWidget::KDMUsersWidget(QWidget *parent, const char *name)
     optoutlv = new KListView( this );
     optoutlv->addColumn( i18n("Hidden Users") );
     optoutlv->setResizeMode( QListView::LastColumn );
-    QWhatsThis::add( optoutlv, i18n("KDM will show all non-checked non-system users.") );
+    QWhatsThis::add( optoutlv, i18n("KDM will show all non-checked non-system users. Entries denoted with '@' are user groups. Checking a group is like checking all users in that group.") );
     wstack->addWidget( optoutlv );
     connect( optoutlv, SIGNAL(clicked( QListViewItem * )),
 	     SLOT(slotUpdateOptOut( QListViewItem * )) );
@@ -200,9 +210,9 @@ void KDMUsersWidget::makeReadOnly()
 {
     leminuid->setReadOnly(true);
     lemaxuid->setReadOnly(true);
-    rbnoneusr->setEnabled(false);
-    rbselusr->setEnabled(false);
-    rballusr->setEnabled(false);
+    cbshowlist->setEnabled(false);
+    cbcomplete->setEnabled(false);
+    cbinverted->setEnabled(false);
     cbusrsrt->setEnabled(false);
     rbadmonly->setEnabled(false);
     rbprefadm->setEnabled(false);
@@ -216,10 +226,12 @@ void KDMUsersWidget::makeReadOnly()
 
 void KDMUsersWidget::slotShowOpts()
 {
-    bool en = !rbnoneusr->isChecked();
+    bool en = cbshowlist->isChecked() || cbcomplete->isChecked();
+    cbinverted->setEnabled( en );
     cbusrsrt->setEnabled( en );
     wstack->setEnabled( en );
-    wstack->raiseWidget( rbselusr->isChecked() ? optinlv : optoutlv );
+    wstack->raiseWidget( cbinverted->isChecked() ? optoutlv : optinlv );
+    en = cbshowlist->isChecked();
     faceGroup->setEnabled( en );
     if (!en) {
 	usercombo->setEnabled( false );
@@ -280,6 +292,8 @@ void KDMUsersWidget::changeUserPix(const QString &pix)
         KMessageBox::sorry(this,
 	    i18n("There was an error saving the image:\n%1")
 		.arg( userpix ) );
+    else
+        chmod( QFile::encodeName( userpix ), 0644 );
 
     slotUserSelected();
 }
@@ -346,10 +360,10 @@ void KDMUsersWidget::save()
     config->writeEntry( "MinShowUID", leminuid->text() );
     config->writeEntry( "MaxShowUID", lemaxuid->text() );
 
+    config->writeEntry( "UserList", cbshowlist->isChecked() );
+    config->writeEntry( "UserCompletion", cbcomplete->isChecked() );
     config->writeEntry( "ShowUsers",
-	rballusr->isChecked() ? "NotHidden" :
-	rbselusr->isChecked() ? "Selected" : "None" );
-
+	cbinverted->isChecked() ? "NotHidden" : "Selected" );
     config->writeEntry( "SortUsers", cbusrsrt->isChecked() );
 
     config->writeEntry( "HiddenUsers", hiddenUsers );
@@ -433,18 +447,13 @@ void KDMUsersWidget::load()
     selectedUsers = config->readListEntry( "SelectedUsers");
     hiddenUsers = config->readListEntry( "HiddenUsers");
 
-    cbusrsrt->setChecked(config->readBoolEntry("SortUsers", true));
-
     leminuid->setText(config->readEntry("MinShowUID", defminuid));
     lemaxuid->setText(config->readEntry("MaxShowUID", defmaxuid));
 
-    QString su = config->readEntry( "ShowUsers" );
-    if (su == QString::fromLatin1("None"))
-	rbnoneusr->setChecked(true);
-    else if (su == QString::fromLatin1("Selected"))
-	rbselusr->setChecked(true);
-    else
-	rballusr->setChecked(true);
+    cbshowlist->setChecked( config->readBoolEntry( "UserList", true ) );
+    cbcomplete->setChecked( config->readBoolEntry( "UserCompletion", false ) );
+    cbinverted->setChecked( config->readEntry( "ShowUsers" ) != "Selected" );
+    cbusrsrt->setChecked(config->readBoolEntry("SortUsers", true));
 
     QString ps = config->readEntry( "FaceSource" );
     if (ps == QString::fromLatin1("UserOnly"))
@@ -464,11 +473,13 @@ void KDMUsersWidget::load()
 
 void KDMUsersWidget::defaults()
 {
-    leminuid->setText(defminuid);
-    lemaxuid->setText(defmaxuid);
-    rballusr->setChecked(true);
-    cbusrsrt->setChecked(true);
-    rbadmonly->setChecked(true);
+    leminuid->setText( defminuid );
+    lemaxuid->setText( defmaxuid );
+    cbshowlist->setChecked( true );
+    cbcomplete->setChecked( false );
+    cbinverted->setChecked( true );
+    cbusrsrt->setChecked( true );
+    rbadmonly->setChecked( true );
     hiddenUsers.clear();
     selectedUsers.clear();
     slotShowOpts();
