@@ -3,11 +3,11 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Class Name    : CFontViewPart
+// Class Name    : KFI::CFontViewPart
 // Author        : Craig Drummond
 // Project       : K Font Installer (kfontinst-kcontrol)
 // Creation Date : 03/08/2002
-// Version       : $Revision: 1.5 $ $Date: 2004/04/30 21:41:38 $
+// Version       : $Revision: 465476 $ $Date: 2005-09-29 23:37:41 +0200 (Do, 29 Sep 2005) $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -26,20 +26,23 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //
 ////////////////////////////////////////////////////////////////////////////////
-// (C) Craig Drummond, 2002, 2003
+// (C) Craig Drummond, 2002, 2003, 2004
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <kparts/part.h>
 
-class CFontPreview;
 class QPushButton;
 class QFrame;
 class QLabel;
 class KIntNumInput;
 class KAction;
-class KToggleAction;
 class KURL;
 
+namespace KFI
+{
+
+class CFontPreview;
+
 class CFontViewPart : public KParts::ReadOnlyPart
 {
     Q_OBJECT
@@ -47,7 +50,7 @@ class CFontViewPart : public KParts::ReadOnlyPart
     public:
 
     CFontViewPart(QWidget *parent=0, const char *name=0);
-    virtual ~CFontViewPart();
+    virtual ~CFontViewPart() {}
 
     protected:
 
@@ -57,11 +60,9 @@ class CFontViewPart : public KParts::ReadOnlyPart
     private slots:
 
     void previewStatus(bool st);
+    void timeout();
     void install();
-    void zoomIn();
-    void zoomOut();
     void changeText();
-    void toggleWaterfall();
 
     private:
 
@@ -71,11 +72,11 @@ class CFontViewPart : public KParts::ReadOnlyPart
                   *itsToolsFrame;
     QLabel        *itsFaceLabel;
     KIntNumInput  *itsFaceSelector;
-    KAction       *itsZoomInAction,
-                  *itsZoomOutAction,
-                  *itsChangeTextAction;
-    KToggleAction *itsToggleWaterfallAction;
+    KAction       *itsChangeTextAction;
     bool          itsShowInstallButton;
+    int           itsFace;
 };
 
+}
+
 #endif
