@@ -9,7 +9,6 @@
 
 
 #include <config.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -19,37 +18,35 @@
 #include <sys/wait.h>
 
 #include <qbuttongroup.h>
-#include <qpushbutton.h>
 #include <qcheckbox.h>
-#include <qslider.h>
-#include <qlayout.h>
-#include <qwhatsthis.h>
-#include <qtimer.h>
-#include <qlabel.h>
-#include <qlistview.h>
 #include <qheader.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qlistview.h>
+#include <qpushbutton.h>
+#include <qslider.h>
+#include <qtimer.h>
+#include <qwhatsthis.h>
 
+#include <dcopclient.h>
+
+#include <kaboutdata.h>
 #include <kapplication.h>
 #include <kdebug.h>
-#include <kdialogbase.h>
-#include <kprocess.h>
-#include <knuminput.h>
-#include <kstandarddirs.h>
-#include <kiconloader.h>
-#include <dcopclient.h>
-#include <kservicegroup.h>
-#include <kgenericfactory.h>
-#include <kwin.h>
 #include <kdialog.h>
+#include <kdialogbase.h>
+#include <kgenericfactory.h>
+#include <kiconloader.h>
+#include <knuminput.h>
+#include <kprocess.h>
+#include <kservicegroup.h>
+#include <kstandarddirs.h>
 
 #include <X11/Xlib.h>
 
 #include "scrnsave.h"
 
-// X11 headers
-#undef Above
-#undef Below
-#undef None
+#include <fixx11h.h>
 
 template class QPtrList<SaverConfig>;
 
@@ -73,244 +70,6 @@ static QString findExe(const QString &exe) {
     return result;
 }
 
-//===========================================================================
-//
-//
-SaverConfig::SaverConfig()
-{
-}
-
-bool SaverConfig::read(const QString &file)
-{
-    KDesktopFile config(file, true);
-    if( !config.tryExec())
-      return false;	
-	mExec = config.readPathEntry("Exec");
-    mName = config.readEntry("Name");
-    mCategory = i18n("Screen saver category", // Must be same in Makefile.am
-                     config.readEntry("X-KDE-Category").utf8());
-
-    if (config.hasActionGroup("Setup"))
-    {
-      config.setActionGroup("Setup");
-      mSetup = config.readPathEntry("Exec");
-    }
-
-    if (config.hasActionGroup("InWindow"))
-    {
-      config.setActionGroup("InWindow");
-      mSaver = config.readPathEntry("Exec");
-    }
-
-    int indx = file.findRev('/');
-    if (indx >= 0) {
-        mFile = file.mid(indx+1);
-    }
-
-    return !mSaver.isEmpty();
-}
-
-//===========================================================================
-//
-int SaverList::compareItems(QPtrCollection::Item item1, QPtrCollection::Item item2)
-{
-    SaverConfig *s1 = (SaverConfig *)item1;
-    SaverConfig *s2 = (SaverConfig *)item2;
-
-    return s1->name().localeAwareCompare(s2->name());
-}
-
-//===========================================================================
-//
-TestWin::TestWin()
-    : QXEmbed(0, 0, WStyle_Customize | WStyle_NoBorder | WX11BypassWM )
-{
-    setFocusPolicy(StrongFocus);
-    KWin::setState( winId(), NET::StaysOnTop );
-}
-
-//===========================================================================
-//
-KScreenSaverAdvancedDialog::KScreenSaverAdvancedDialog(QWidget *parent, const char* name)
- : KDialogBase( parent, name, true, i18n( "Advanced Options" ),
-                Ok | Cancel, Ok, true )
-{
-    readSettings();
-
-    QWidget* w = new QWidget( this );
-    w->setMinimumWidth( 250 );
-    QBoxLayout* mainLayout = new QVBoxLayout( w, 0, KDialog::spacingHint() );
-    setMainWidget( w );
-
-    // Autolock
-    QGroupBox* autoLockGroup = new QGroupBox( i18n("Auto Lock"), w );
-    autoLockGroup->setColumnLayout( 0, Qt::Vertical );
-    mainLayout->addWidget( autoLockGroup );
-    QBoxLayout* autoLockLayout = new QVBoxLayout( autoLockGroup->layout(),
-        KDialog::spacingHint() );
-
-    m_topLeftCorner = new QCheckBox( i18n("Top-left corner"), autoLockGroup);
-    autoLockLayout->addWidget( m_topLeftCorner );
-    m_topLeftCorner->setChecked(mTopLeftCorner);
-    connect( m_topLeftCorner, SIGNAL( toggled( bool ) ),
-            this, SLOT( slotChangeTopLeftCorner( bool ) ) );
-
-    m_topRightCorner = new QCheckBox( i18n("Top-right corner"),
-        autoLockGroup );
-    autoLockLayout->addWidget( m_topRightCorner );
-    m_topRightCorner->setChecked(mTopRightCorner);
-    connect( m_topRightCorner, SIGNAL( toggled( bool ) ),
-         this, SLOT( slotChangeTopRightCorner( bool ) ) );
-
-    m_bottomLeftCorner = new QCheckBox( i18n("Bottom-left corner"),
-        autoLockGroup );
-    autoLockLayout->addWidget( m_bottomLeftCorner );
-    m_bottomLeftCorner->setChecked(mBottomLeftCorner);
-    connect( m_bottomLeftCorner, SIGNAL( toggled( bool ) ),
-         this, SLOT( slotChangeBottomLeftCorner( bool ) ) );
-
-    m_bottomRightCorner = new QCheckBox( i18n("Bottom-right corner"),
-        autoLockGroup );
-    autoLockLayout->addWidget( m_bottomRightCorner );
-    m_bottomRightCorner->setChecked(mBottomRightCorner);
-    connect( m_bottomRightCorner, SIGNAL( toggled( bool ) ),
-         this, SLOT( slotChangeBottomRightCorner( bool ) ) );
-
-    // Priority
-    QGroupBox* priorityGroup = new QGroupBox( i18n("&Priority"), w );
-    priorityGroup->setColumnLayout( 0, Qt::Horizontal );
-    mainLayout->addWidget( priorityGroup );
-
-    QGridLayout *gl = new QGridLayout(priorityGroup->layout(), 2, 3);
-    gl->setColStretch( 1, 10 );
-
-    mPrioritySlider = new QSlider(QSlider::Horizontal, priorityGroup);
-    mPrioritySlider->setRange(0, 19);
-    mPrioritySlider->setSteps(1, 5);
-    mPrioritySlider->setTickmarks(QSlider::Below);
-    mPrioritySlider->setValue(19 - mPriority);
-    connect(mPrioritySlider, SIGNAL( valueChanged(int)),
-        SLOT(slotPriorityChanged(int)));
-    gl->addMultiCellWidget(mPrioritySlider, 0, 0, 0, 2);
-    QWhatsThis::add( mPrioritySlider, i18n("Use this slider to change the"
-      " processing priority for the screen saver over other jobs that are"
-      " being executed in the background. For a processor-intensive screen"
-      " saver, setting a higher priority may make the display smoother at"
-      " the expense of other jobs.") );
-
-#ifndef HAVE_SETPRIORITY
-    lbl->setEnabled(false);
-    mPrioritySlider->setEnabled(false);
-#endif
-
-    QLabel* lbl = new QLabel(i18n("Low Priority", "Low"), priorityGroup);
-    gl->addWidget(lbl, 1, 0);
-
-#ifndef HAVE_SETPRIORITY
-    lbl->setEnabled(false);
-#endif
-
-    lbl = new QLabel(i18n("High Priority", "High"), priorityGroup);
-    gl->addWidget(lbl, 1, 2);
-
-#ifndef HAVE_SETPRIORITY
-    lbl->setEnabled(false);
-#endif
-
-    mainLayout->addStretch(10);
-}
-
-
-//===========================================================================
-//
-void KScreenSaverAdvancedDialog::readSettings()
-{
-    KConfig *config = new KConfig( "kdesktoprc");
-    config->setGroup( "ScreenSaver" );
-
-    mPriority = config->readNumEntry("Priority", 19);
-    if (mPriority < 0) mPriority = 0;
-    if (mPriority > 19) mPriority = 19;
-
-    mTopLeftCorner = config->readBoolEntry("LockCornerTopLeft", false);
-    mTopRightCorner = config->readBoolEntry("LockCornerTopRight", false) ;
-    mBottomLeftCorner = config->readBoolEntry("LockCornerBottomLeft", false);
-    mBottomRightCorner = config->readBoolEntry("LockCornerBottomRight", false);
-    mChanged = false;
-    delete config;
-}
-
-
-//===========================================================================
-//
-void KScreenSaverAdvancedDialog::slotOk()
-{
-    if ( mChanged ) {
-        KConfig *config = new KConfig( "kdesktoprc");
-        config->setGroup( "ScreenSaver" );
-
-        config->writeEntry("Priority", mPriority);
-        config->writeEntry(
-            "LockCornerTopLeft", m_topLeftCorner->isChecked());
-        config->writeEntry(
-            "LockCornerBottomLeft", m_bottomLeftCorner->isChecked());
-        config->writeEntry(
-            "LockCornerTopRight", m_topRightCorner->isChecked());
-        config->writeEntry(
-            "LockCornerBottomRight", m_bottomRightCorner->isChecked());
-        config->sync();
-        delete config;
-    }
-    accept();
-}
-
-
-//---------------------------------------------------------------------------
-//
-void KScreenSaverAdvancedDialog::slotPriorityChanged( int val )
-{
-    if (val == mPriority)
-    return;
-
-    mPriority = 19 - val;
-    if (mPriority > 19)
-    mPriority = 19;
-    else if (mPriority < 0)
-    mPriority = 0;
-
-    mChanged = true;
-}
-
-
-//===========================================================================
-//
-void KScreenSaverAdvancedDialog::slotChangeBottomRightCorner( bool b)
-{
-    mBottomRightCorner = b;
-    mChanged = true;
-}
-
-void KScreenSaverAdvancedDialog::slotChangeBottomLeftCorner( bool b)
-{
-    mBottomLeftCorner = b;
-    mChanged = true;
-}
-
-void KScreenSaverAdvancedDialog::slotChangeTopRightCorner( bool b)
-{
-    mTopRightCorner = b;
-    mChanged = true;
-}
-
-void KScreenSaverAdvancedDialog::slotChangeTopLeftCorner( bool b)
-{
-    mTopLeftCorner = b;
-    mChanged = true;
-}
-
-//===========================================================================
-//
-
 KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&)
     : KCModule(KSSFactory::instance(), parent, name)
 {
@@ -327,6 +86,18 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
                                      KGlobal::dirs()->kde_default("apps") +
                                      "apps/ScreenSavers/");
 
+    setQuickHelp( i18n("<h1>Screen Saver</h1> This module allows you to enable and"
+       " configure a screen saver. Note that you can enable a screen saver"
+       " even if you have power saving features enabled for your display.<p>"
+       " Besides providing an endless variety of entertainment and"
+       " preventing monitor burn-in, a screen saver also gives you a simple"
+       " way to lock your display if you are going to leave it unattended"
+       " for a while. If you want the screen saver to lock the session, make sure you enable"
+       " the \"Require password\" feature of the screen saver; if you do not, you can still"
+       " explicitly lock the session using the desktop's \"Lock Session\" action."));
+
+    setButtons( KCModule::Help | KCModule::Default | KCModule::Apply );
+
     // Add KDE specific screensaver path
     QString relPath="System/ScreenSavers/";
     KServiceGroup::Ptr servGroup = KServiceGroup::baseGroup( "screensavers" );
@@ -373,24 +144,20 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
     mSelected = -1;
     groupLayout->addWidget( mSaverListView, 10 );
     connect( mSaverListView, SIGNAL(doubleClicked ( QListViewItem *)), this, SLOT( slotSetup()));
-    QWhatsThis::add( mSaverListView, i18n("This is a list of the available"
-      " screen savers. Select the one you want to use.") );
+    QWhatsThis::add( mSaverListView, i18n("Select the screen saver to use.") );
 
     QBoxLayout* hlay = new QHBoxLayout(groupLayout, KDialog::spacingHint());
     mSetupBt = new QPushButton( i18n("&Setup..."), mSaverGroup );
     connect( mSetupBt, SIGNAL( clicked() ), SLOT( slotSetup() ) );
     mSetupBt->setEnabled(false);
     hlay->addWidget( mSetupBt );
-    QWhatsThis::add( mSetupBt, i18n("If the screen saver you selected has"
-      " customizable features, you can set them up by clicking this button.") );
+    QWhatsThis::add( mSetupBt, i18n("Configure the screen saver's options, if any.") );
 
     mTestBt = new QPushButton( i18n("&Test"), mSaverGroup );
     connect( mTestBt, SIGNAL( clicked() ), SLOT( slotTest() ) );
     mTestBt->setEnabled(false);
     hlay->addWidget( mTestBt );
-    QWhatsThis::add( mTestBt, i18n("You can try out the screen saver by clicking"
-      " this button. (Also, the preview image shows you what the screen saver"
-      " will look like.)") );
+    QWhatsThis::add( mTestBt, i18n("Show a full screen preview of the screen saver.") );
 
     mSettingsGroup = new QGroupBox( i18n("Settings"), this );
     mSettingsGroup->setColumnLayout( 0, Qt::Vertical );
@@ -402,10 +169,7 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
         "Start a&utomatically"), mSettingsGroup);
     mEnabledCheckBox->setChecked(mEnabled);
     QWhatsThis::add( mEnabledCheckBox, i18n(
-        "When you check this option, the selected screen saver "
-        "will be started automatically after a certain number "
-        "of minutes of inactivity. "
-        "This timeout period can be set using the spinbox below.") );
+        "Automatically start the screen saver after a period of inactivity.") );
     connect(mEnabledCheckBox, SIGNAL(toggled(bool)),
             this, SLOT(slotEnable(bool)));
     groupLayout->addWidget(mEnabledCheckBox);
@@ -429,10 +193,8 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
     hbox->addWidget(mWaitEdit);
     hbox->addStretch(1);
     QString wtstr = i18n(
-        "Choose the period of inactivity "
-        "after which the screen saver should start. "
-        "To prevent the screen saver from automatically starting, "
-        "uncheck the above option.");
+        "The period of inactivity "
+        "after which the screen saver should start.");
     QWhatsThis::add( mActivateLbl, wtstr );
     QWhatsThis::add( mWaitEdit, wtstr );
 
@@ -444,20 +206,15 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
              this, SLOT( slotLock( bool ) ) );
     groupLayout->addWidget(mLockCheckBox);
     QWhatsThis::add( mLockCheckBox, i18n(
-        "If you check this option, the display will be locked after "
-        "the screen saver has been started. To restore the "
-        "display, enter your account password at the prompt. "
-        "You can specify in the spinbox below a delay "
-        "after which the display will actually be locked."  ) );
+        "Prevent potential unauthorized use by requiring a password"
+        " to stop the screen saver.") );
     hbox = new QHBoxLayout();
     groupLayout->addLayout(hbox);
     hbox->addSpacing(30);
     mLockLbl = new QLabel(i18n("After:"), mSettingsGroup);
     mLockLbl->setEnabled(mEnabled && mLock);
     QWhatsThis::add( mLockLbl, i18n(
-        "The period after which the display will be locked."
-        "To prevent the display from being locked, disable the "
-        "option above." ) );
+        "The amount of time, after the screen saver has started, to ask for the unlock password.") );
     hbox->addWidget(mLockLbl);
     mWaitLockEdit = new QSpinBox(mSettingsGroup);
     mWaitLockEdit->setSteps(1, 10);
@@ -505,12 +262,12 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
     mMonitorLabel->setPixmap( QPixmap(locate("data",
                          "kcontrol/pics/monitor.png")));
     rightColumnLayout->addWidget(mMonitorLabel, 0);
-    QWhatsThis::add( mMonitorLabel, i18n("Here you can see a preview of the selected screen saver.") );
+    QWhatsThis::add( mMonitorLabel, i18n("A preview of the selected screen saver.") );
 
     QBoxLayout* advancedLayout = new QHBoxLayout( rightColumnLayout, 3 );
     advancedLayout->addWidget( new QWidget( this ) );
     QPushButton* advancedBt = new QPushButton(
-        i18n( "Advanced &Options..." ), this, "advancedBtn" );
+        i18n( "Advanced &Options" ), this, "advancedBtn" );
     advancedBt->setSizePolicy( QSizePolicy(
         QSizePolicy::Fixed, QSizePolicy::Fixed) );
     connect( advancedBt, SIGNAL( clicked() ),
@@ -535,6 +292,17 @@ KScreenSaver::KScreenSaver(QWidget *parent, const char *name, const QStringList&
     mLoadTimer->start( 100 );
     mChanged = false;
     emit changed(false);
+
+    KAboutData *about =
+    new KAboutData(I18N_NOOP("kcmscreensaver"), I18N_NOOP("KDE Screen Saver Control Module"),
+                  0, 0, KAboutData::License_GPL,
+                  I18N_NOOP("(c) 1997-2002 Martin R. Jones\n"
+                  "(c) 2003-2004 Chris Howells"));
+    about->addAuthor("Chris Howells", 0, "howells@kde.org");
+    about->addAuthor("Martin R. Jones", 0, "jones@kde.org");
+
+    setAboutData( about );
+
 }
 
 //---------------------------------------------------------------------------
@@ -564,14 +332,6 @@ void KScreenSaver::keyPressEvent( QKeyEvent *)
     if ( mTesting )
 	slotStopTest();
 }
-
-//---------------------------------------------------------------------------
-//
-int KScreenSaver::buttons()
-{
-    return KCModule::Help | KCModule::Default | KCModule::Apply;
-}
-
 //---------------------------------------------------------------------------
 //
 KScreenSaver::~KScreenSaver()
@@ -903,12 +663,23 @@ void KScreenSaver::slotScreenSaver(QListViewItem *item)
 
     int i = 0, indx = -1;
     for (SaverConfig* saver = mSaverList.first(); saver != 0; saver = mSaverList.next()) {
-        if (saver->name() == item->text (0))
+        if ( item->parent() )
         {
-            indx = i;
-            break;
+            if (  item->parent()->text( 0 ) == saver->category() && saver->name() == item->text (0))
+            {
+                indx = i;
+                break;
+            }
         }
-        i++;
+        else
+        {
+            if (  saver->name() == item->text (0) )
+            {
+                indx = i;
+                break;
+            }
+        }        
+		i++;
     }
     if (indx == -1) {
         mSelected = -1;
@@ -1121,19 +892,4 @@ void KScreenSaver::slotSetupDone(KProcess *)
     emit changed(true);
 }
 
-//---------------------------------------------------------------------------
-//
-QString KScreenSaver::quickHelp() const
-{
-    return i18n("<h1>Screen Saver</h1> This module allows you to enable and"
-       " configure a screen saver. Note that you can enable a screen saver"
-       " even if you have power saving features enabled for your display.<p>"
-       " Besides providing an endless variety of entertainment and"
-       " preventing monitor burn-in, a screen saver also gives you a simple"
-       " way to lock your display if you are going to leave it unattended"
-       " for a while. If you want the screen saver to lock the session, make sure you enable"
-       " the \"Require password\" feature of the screen saver; if you do not, you can still"
-       " explicitly lock the session using the desktop's \"Lock Session\" action.");
-}
-
 #include "scrnsave.moc"
