@@ -42,12 +42,12 @@ class KActionCollection;
 
 class KRootWidget : public QObject
 {
+    Q_OBJECT
 public:
    KRootWidget();
    bool eventFilter( QObject *, QEvent * e );
-   void setWheelSwitchEnabled( bool enable );
-private:
-   bool m_bWheelSwitchesWorkspace;
+signals:
+   void wheelRolled( int delta );
 };
 
 /**
@@ -58,8 +58,11 @@ private:
 class KDesktop : public QWidget, virtual public KDesktopIface
 {
   Q_OBJECT
+
 public:
 
+  enum WheelDirection { Forward = 0, Reverse };
+
   KDesktop(bool x_root_hack, bool auto_start, bool wait_for_kded );
   ~KDesktop();
 
@@ -73,10 +76,13 @@ public:
 
   virtual void configure();
   virtual void popupExecuteCommand();
+  virtual void popupExecuteCommand(const QString& content);
   virtual void refresh();
   virtual void logout();
   virtual void clearCommandHistory();
 
+  virtual void switchDesktops( int delta );
+
   void logout( KApplication::ShutdownConfirm confirm, KApplication::ShutdownType sdtype );
 
   KWinModule* kwinModule() const { return m_pKwinmodule; }
@@ -114,6 +120,8 @@ private slots:
 
   void slotShowWindowList();
 
+  void slotSwitchUser();
+
   void slotLogout();
   void slotLogoutNoCnf();
   void slotHaltNoCnf();
@@ -136,6 +144,9 @@ private slots:
 
   void updateWorkArea();
 
+  /** Connected to KDIconView and KRootWidget  */
+  void slotSwitchDesktops(int delta);
+
 protected:
   void initConfig();
   void initRoot();
@@ -148,10 +159,10 @@ protected:
   virtual bool isIconsEnabled() { return m_bDesktopEnabled; }
   virtual void setIconsEnabled( bool enable );
   virtual bool event ( QEvent * e );
- 
+
 private slots:
   void desktopResized();
-  
+
 private:
 
   KGlobalAccel *keys;
@@ -178,9 +189,23 @@ private:
 
   /** Wait for kded to finish building database? */
   bool m_bWaitForKded;
-  
+
   /** Desktop enabled / disabled **/
   bool m_bDesktopEnabled;
+
+  /** Whether or not to switch desktops when mouse wheel is rolled */
+  bool m_bWheelSwitchesWorkspace;
+
+  /** Default mouse wheel direction (Fwd means mwheel up switches to
+      lower desktop)
+  */
+  static const WheelDirection m_eDefaultWheelDirection = Forward;
+
+  /** Mouse wheel/desktop switching direction */
+  static WheelDirection m_eWheelDirection;
+
+  /** Possible values for "kdesktoprc"->"Mouse Buttons"->"WheelDirection" */
+  static const char* m_wheelDirectionStrings[2];
 };
 
 #endif
