@@ -15,11 +15,14 @@
 //crashes (e.g. because it's set to multiple wallpapers and
 //some image will be corrupted).
 
-
 #include <config.h>
 
 #include "lockprocess.h"
 #include "lockdlg.h"
+#include "autologout.h"
+#include "kdesktopsettings.h"
+
+#include <dmctl.h>
 
 #include <kstandarddirs.h>
 #include <kapplication.h>
@@ -31,6 +34,8 @@
 #include <klibloader.h>
 #include <kpushbutton.h>
 #include <kstdguiitem.h>
+#include <kpixmapeffect.h>
+#include <kpixmap.h>
 
 #include <qframe.h>
 #include <qlabel.h>
@@ -41,6 +46,8 @@
 #include <qsocketnotifier.h>
 #include <qvaluevector.h>
 
+#include <qdatetime.h>
+
 #include <stdlib.h>
 #include <assert.h>
 #include <signal.h>
@@ -53,6 +60,7 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
+#include <X11/Intrinsic.h>
 
 #ifdef HAVE_DPMS
 extern "C" {
@@ -72,7 +80,12 @@ Status DPMSInfo ( Display *, CARD16 *, BOOL * );
 #include <X11/extensions/xf86misc.h>
 #endif
 
+#ifdef HAVE_GLXCHOOSEVISUAL
+#include <GL/glx.h>
+#endif
+
 #define LOCK_GRACE_DEFAULT          5000
+#define AUTOLOGOUT_DEFAULT          600
 
 static Window gVRoot = 0;
 static Window gVRootData = 0;
@@ -82,17 +95,19 @@ static Atom   gXA_SCREENSAVER_VERSION;
 //===========================================================================
 //
 // Screen saver handling process.  Handles screensaver window,
-// starting screensaver hacks, and password entry.
+// starting screensaver hacks, and password entry.f
 //
 LockProcess::LockProcess(bool child, bool useBlankOnly)
     : QWidget(0L, "saver window", WX11BypassWM),
+      mOpenGLVisual(0),
       child_saver(child),
       mParent(0),
       mUseBlankOnly(useBlankOnly),
       mSuspended(false),
       mVisibility(false),
       mRestoreXF86Lock(false),
-      mForbidden(false)
+      mForbidden(false),
+      mAutoLogout(false)
 {
     setupSignals();
 
@@ -128,10 +143,6 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
     gXA_VROOT = XInternAtom (qt_xdisplay(), "__SWM_VROOT", False);
     gXA_SCREENSAVER_VERSION = XInternAtom (qt_xdisplay(), "_SCREENSAVER_VERSION", False);
 
-    XWindowAttributes attrs;
-    XGetWindowAttributes(qt_xdisplay(), winId(), &attrs);
-    mColorMap = attrs.colormap;
-
     connect(&mHackProc, SIGNAL(processExited(KProcess *)),
                         SLOT(hackExited(KProcess *)));
 
@@ -140,8 +151,6 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
     QStringList dmopt =
         QStringList::split(QChar(','),
                             QString::fromLatin1( ::getenv( "XDM_MANAGED" )));
-    if (dmopt.findIndex( "rsvd" ) >= 0)
-        mXdmFifoName = dmopt.first();
     for (QStringList::ConstIterator it = dmopt.begin(); it != dmopt.end(); ++it)
         if ((*it).startsWith("method="))
             mMethod = (*it).mid(7);
@@ -185,6 +194,16 @@ static void sigterm_handler(int)
     ::write( sigterm_pipe[1], &tmp, 1);
 }
 
+void LockProcess::timerEvent(QTimerEvent *ev)
+{
+	if (ev->timerId() == mAutoLogoutTimerId)
+	{
+		killTimer(mAutoLogoutTimerId);
+		AutoLogout autologout(this);
+		execDialog(&autologout);
+	}
+}
+
 void LockProcess::setupSignals()
 {
     // ignore SIGHUP
@@ -284,13 +303,9 @@ void LockProcess::quitSaver()
 void LockProcess::configure()
 {
     // the configuration is stored in kdesktop's config file
-    KConfig config( "kdesktoprc", true );
-
-    config.setGroup("ScreenSaver");
-
-    if(config.readBoolEntry("Lock", false))
+    if( KDesktopSettings::lock() )
     {
-        mLockGrace = config.readNumEntry("LockGrace", LOCK_GRACE_DEFAULT);
+        mLockGrace = KDesktopSettings::lockGrace();
         if (mLockGrace < 0)
             mLockGrace = 0;
         else if (mLockGrace > 300000)
@@ -299,26 +314,33 @@ void LockProcess::configure()
     else
         mLockGrace = -1;
 
+    if ( KDesktopSettings::autoLogout() )
+    {
+        mAutoLogout = true;
+        mAutoLogoutTimeout = KDesktopSettings::autoLogoutTimeout();
+        mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000); // in milliseconds
+    }
+
 #ifdef HAVE_DPMS
     //if the user  decided that the screensaver should run independent from
     //dpms, we shouldn't check for it, aleXXX
-    mDPMSDepend = config.readBoolEntry("DPMS-dependent", false);
+    mDPMSDepend = KDesktopSettings::dpmsDependent();
 #endif
 
-    mPriority = config.readNumEntry("Priority", 19);
+    mPriority = KDesktopSettings::priority();
     if (mPriority < 0) mPriority = 0;
     if (mPriority > 19) mPriority = 19;
 
-    mSaver = config.readEntry("Saver");
+    mSaver = KDesktopSettings::saver();
     if (mSaver.isEmpty() || mUseBlankOnly)
         mSaver = "KBlankscreen.desktop";
 
     readSaver();
 
-    mPlugins = config.readListEntry("PluginsUnlock");
+    mPlugins = KDesktopSettings::pluginsUnlock();
     if (mPlugins.isEmpty())
         mPlugins = QStringList("classic");
-    mPluginOptions = config.readListEntry("PluginOptions");
+    mPluginOptions = KDesktopSettings::pluginOptions();
 }
 
 //---------------------------------------------------------------------------
@@ -350,6 +372,10 @@ void LockProcess::readSaver()
 				kdDebug(1204) << "Screensaver is type OpenGL and OpenGL is forbidden" << endl;
 				mForbidden = true;
 			}
+			if (saverTypes[i] == "OpenGL")
+			{
+				mOpenGLVisual = true;
+			}
 		}
 	}
 
@@ -369,8 +395,29 @@ void LockProcess::readSaver()
 //
 void LockProcess::createSaverWindow()
 {
-    // We only create the window once, but we reset its attributes every
-    // time.
+    Visual* visual = CopyFromParent;
+    XSetWindowAttributes attrs;
+    int flags = 0;
+#ifdef HAVE_GLXCHOOSEVISUAL
+    if( mOpenGLVisual )
+    {
+        int attribs[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, x11Depth(), None };
+        if( XVisualInfo* i = glXChooseVisual( x11Display(), x11Screen(), attribs ))
+        {
+            visual = i->visual;
+            static Colormap colormap = 0;
+            if( colormap != 0 )
+                XFreeColormap( x11Display(), colormap );
+            colormap = XCreateColormap( x11Display(), RootWindow( x11Display(), x11Screen()), visual, AllocNone );
+            attrs.colormap = colormap;
+            flags |= CWColormap;
+            XFree( i );
+        }
+    }
+#endif
+    Window w = XCreateWindow( x11Display(), RootWindow( x11Display(), x11Screen()),
+        x(), y(), width(), height(), 0, x11Depth(), InputOutput, visual, flags, &attrs );
+    create( w );
 
     // Some xscreensaver hacks check for this property
     const char *version = "KDE 2.0";
@@ -379,19 +426,10 @@ void LockProcess::createSaverWindow()
                      (unsigned char *) version, strlen(version));
 
     XSetWindowAttributes attr;
-    if (mColorMap != None)
-    {
-        attr.colormap = mColorMap;
-    }
-    else
-    {
-        attr.colormap = DefaultColormapOfScreen(
-                                ScreenOfDisplay(qt_xdisplay(), qt_xscreen()));
-    }
     attr.event_mask = KeyPressMask | ButtonPressMask | PointerMotionMask |
                         VisibilityChangeMask | ExposureMask;
     XChangeWindowAttributes(qt_xdisplay(), winId(),
-                            CWEventMask | CWColormap, &attr);
+                            CWEventMask, &attr);
 
     // erase();
 
@@ -585,25 +623,6 @@ void LockProcess::ungrabInput()
     unlockXF86();
 }
 
-//---------------------------------------------------------------------------
-//
-// Send KDM (or XDM, if it gets adapted) a command
-//
-void LockProcess::xdmFifoCmd(const char *cmd)
-{
-    QFile fifo(mXdmFifoName);
-    if (fifo.open(IO_WriteOnly | IO_Raw)) {
-        fifo.writeBlock( cmd, ::strlen(cmd) );
-        fifo.close();
-    }
-}
-
-void LockProcess::xdmFifoLockCmd(const char *cmd)
-{
-    if (!mXdmFifoName.isNull() && mLocked)
-        xdmFifoCmd(cmd);
-}
-
 //---------------------------------------------------------------------------
 //
 // Start the screen saver.
@@ -631,9 +650,7 @@ bool LockProcess::startSaver()
     raise();
     XSync(qt_xdisplay(), False);
     setVRoot( winId(), winId() );
-    if (!startHack())
-        setBackgroundColor(black); // failed to start a hack.  Just show a blank screen
-
+    startHack();
     return true;
 }
 
@@ -649,7 +666,8 @@ void LockProcess::stopSaver()
     hideSaverWindow();
     mVisibility = false;
     if (!child_saver) {
-        xdmFifoLockCmd("unlock\n");
+        if (mLocked)
+            DM().setLock( false );
         ungrabInput();
         const char *out = "GOAWAY!";
         for (QValueList<int>::ConstIterator it = child_sockets.begin(); it != child_sockets.end(); ++it)
@@ -715,18 +733,18 @@ bool LockProcess::startLock()
         }
         kdDebug(1204) << "GreeterPlugin " << *it << " (" << plugin.info->method << ", " << plugin.info->name << ") loaded" << endl;
         greetPlugin = plugin;
-	KGlobal::locale()->insertCatalogue( "kdmgreet" );
 	mLocked = true;
-	xdmFifoLockCmd("lock\n");
+	DM().setLock( true );
 	return true;
     }
     cantLock( i18n("No appropriate greeter plugin configured.") );
     return false;
 }
 
-
 //---------------------------------------------------------------------------
 //
+
+
 bool LockProcess::startHack()
 {
     if (mSaverExec.isEmpty())
@@ -764,15 +782,19 @@ bool LockProcess::startHack()
 
 	if (!mForbidden)
 	{
+
 		if (mHackProc.start() == true)
 		{
 #ifdef HAVE_SETPRIORITY
 			setpriority(PRIO_PROCESS, mHackProc.pid(), mPriority);
 #endif
-			return true;
+ 		        //bitBlt(this, 0, 0, &mOriginal);
+	                return true;
 		}
 	}
-	else // we aren't allowed to start the specified screensaver according to the kiosk restrictions
+	else
+	// we aren't allowed to start the specified screensaver either because it didn't run for some reason
+	// according to the kiosk restrictions forbid it
 	{
 		setBackgroundColor(black);
 	}
@@ -832,8 +854,8 @@ void LockProcess::resume()
 //
 bool LockProcess::checkPass()
 {
-    PasswordDlg passDlg( this, &greetPlugin, !mXdmFifoName.isNull());
-    connect(&passDlg, SIGNAL(startNewSession()), SLOT(startNewSession()));
+    killTimer(mAutoLogoutTimerId);
+    PasswordDlg passDlg( this, &greetPlugin);
 
     return execDialog( &passDlg ) == QDialog::Accepted;
 }
@@ -881,6 +903,19 @@ int LockProcess::execDialog( QDialog *dlg )
     return rt;
 }
 
+void LockProcess::preparePopup()
+{
+    QWidget *dlg = (QWidget *)sender();
+    mDialogs.prepend( dlg );
+    fakeFocusIn( dlg->winId() );
+}
+
+void LockProcess::cleanupPopup()
+{
+    QWidget *dlg = (QWidget *)sender();
+    mDialogs.remove( dlg );
+    fakeFocusIn( mDialogs.first()->winId() );
+}
 
 //---------------------------------------------------------------------------
 //
@@ -901,6 +936,11 @@ bool LockProcess::x11Event(XEvent *event)
                 stopSaver();
                 kapp->quit();
             }
+            else if (mAutoLogout) // we need to restart the auto logout countdown
+            {
+                killTimer(mAutoLogoutTimerId);
+                mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout);
+            }
             mBusy = false;
             return true;
 
@@ -976,11 +1016,6 @@ void LockProcess::stayOnTop()
         XRaiseWindow(qt_xdisplay(), winId());
 }
 
-void LockProcess::startNewSession()
-{
-    xdmFifoCmd("reserve\n");
-}
-
 void LockProcess::checkDPMSActive()
 {
 #ifdef HAVE_DPMS
@@ -1063,5 +1098,4 @@ void LockProcess::msgBox( QMessageBox::Icon type, const QString &txt )
     execDialog( &box );
 }
 
-
 #include "lockprocess.moc"
