@@ -63,6 +63,7 @@
 #include <kstdguiitem.h>
 #include <kmimetype.h>
 #include <kurifilter.h>
+#include <kcompletionbox.h>
 
 #include "minicli.moc"
 #include "minicli_ui.h"
@@ -87,14 +88,32 @@ Minicli::Minicli( QWidget *parent, const char *name)
   m_dlg->cbCommand->setDuplicatesEnabled( false );
   m_dlg->cbCommand->setTrapReturnKey( true );
 
+  // IMPORTANT NOTE: the buttons in the minicli are QPushButtons, not KPushButtons
+  // this is because KPushButtons install a tooltip automatically which, via
+  // QTipManager's (properly written, even) event filter on the qApp, interferes
+  // with event filters installed on the root window (qApp->desktop()) which prevents
+  // the user from being able to access desktop menus in certain configurations
+  // (e.g. icons turned off)
+  // so the KGuiItems are handled manually and QPushButtons are users.
+  // see BR#110607
+
   // Options button...
-  m_dlg->pbOptions->setGuiItem (KGuiItem( i18n("&Options >>"), "configure" ));
+  KGuiItem guiItem = KGuiItem( i18n("&Options >>"), "configure" );
+  m_dlg->pbOptions->setIconSet(guiItem.iconSet());
+  m_dlg->pbOptions->setText(guiItem.text());
+  QWhatsThis::add(m_dlg->pbOptions, guiItem.whatsThis());
 
   // Run button...
-  m_dlg->pbRun->setGuiItem (KGuiItem( i18n("&Run"), "run" ));
+  guiItem = KGuiItem( i18n("&Run"), "run" );
+  m_dlg->pbRun->setIconSet(guiItem.iconSet());
+  m_dlg->pbRun->setText(guiItem.text());
+  QWhatsThis::add(m_dlg->pbRun, guiItem.whatsThis());
 
   // Cancel button...
-  m_dlg->pbCancel->setGuiItem ( KStdGuiItem::cancel() );
+  guiItem = KStdGuiItem::cancel();
+  m_dlg->pbCancel->setIconSet(guiItem.iconSet());
+  m_dlg->pbCancel->setText(guiItem.text());
+  QWhatsThis::add(m_dlg->pbCancel, guiItem.whatsThis());
 
   if (!kapp->authorize("shell_access"))
     m_dlg->pbOptions->hide();
@@ -129,7 +148,8 @@ Minicli::Minicli( QWidget *parent, const char *name)
 
   connect( m_dlg->cbCommand, SIGNAL( textChanged( const QString& ) ),
            SLOT( slotCmdChanged(const QString&) ) );
-  connect( m_dlg->cbCommand, SIGNAL( activated( const QString& ) ),
+
+  connect( m_dlg->cbCommand, SIGNAL( returnPressed() ),
            m_dlg->pbRun, SLOT( animateClick() ) );
 
   // Advanced group box...
@@ -212,6 +232,10 @@ void Minicli::loadConfig()
   int mode = KDesktopSettings::completionMode();
   m_dlg->cbCommand->setCompletionMode( (KGlobalSettings::Completion) mode );
 
+  KCompletionBox* box = m_dlg->cbCommand->completionBox();
+  if (box)
+    box->setActivateOnSelect( false );
+
   m_finalFilters = KURIFilter::self()->pluginNames();
   m_finalFilters.remove("kuriikwsfilter");
 
@@ -235,6 +259,7 @@ void Minicli::loadConfig()
   m_dlg->leUsername->setCompletionObject(completion, true);
   m_dlg->leUsername->setCompletionMode(KGlobalSettings::completionMode());
   m_dlg->leUsername->setAutoDeleteCompletionObject( true );
+
 }
 
 void Minicli::saveConfig()
