@@ -1,6 +1,6 @@
 /* kastaskpopup.cpp
 **
-** Copyright (C) 2001-2003 Richard Moore <rich@kde.org>
+** Copyright (C) 2001-2004 Richard Moore <rich@kde.org>
 ** Contributor: Mosfet
 **     All rights reserved.
 **
@@ -51,17 +51,21 @@
 /*
 ** Bug reports and questions can be sent to kde-devel@kde.org
 */
-#include <kpixmapeffect.h>
 #include <qpainter.h>
 #include <qapplication.h>
+
+#include <kdebug.h>
 #include <kglobalsettings.h>
+#include <kpixmapeffect.h>
 
 #include <taskmanager.h>
 
 #include "kastaskitem.h"
-#include "kastaskpopup.h"
 #include "kastasker.h"
 
+#include "kastaskpopup.h"
+#include "kastaskpopup.moc"
+
 static const int TITLE_HEIGHT = 13;
 
 KasTaskPopup::KasTaskPopup( KasTaskItem *item, const char *name )
@@ -74,15 +78,16 @@ KasTaskPopup::KasTaskPopup( KasTaskItem *item, const char *name )
 
     QString text = item->task()->visibleIconicName();
     if ( item->kasbar()->thumbnailsEnabled() && item->task()->hasThumbnail() ) {
-        resize( item->task()->thumbnail().width() + 2, 
-		TITLE_HEIGHT + item->task()->thumbnail().height() + 2 );
         titleBg.resize( width(), TITLE_HEIGHT );
+
+        setFixedSize( item->task()->thumbnail().width() + 2, 
+		      TITLE_HEIGHT + item->task()->thumbnail().height() + 2 );
     }
     else {
         int w = fontMetrics().width( text ) + 4;
         int h = TITLE_HEIGHT + 1;
-        resize( w, h );
         titleBg.resize( w, h );
+        setFixedSize( w, h );
     }
 
     KPixmapEffect::gradient( titleBg, 
@@ -132,4 +137,3 @@ void KasTaskPopup::paintEvent( QPaintEvent * )
     p.drawRect( 0, 0, width(), height() );
 }
 
-#include "kastaskpopup.moc"
