@@ -26,6 +26,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <qpixmap.h>
 #include <qimage.h>
 
+#include <dcopclient.h>
 #include <kapplication.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
@@ -34,20 +35,24 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kmimetype.h>
+#include <kprocess.h>
 #include <krun.h>
 #include <kservicegroup.h>
 #include <ksycoca.h>
 #include <ksycocaentry.h>
 #include <kservice.h>
 #include <kurldrag.h>
+#include <kio/job.h>
 
+#include "global.h"
 #include "kicker.h"
+#include "kickerSettings.h"
+#include "menumanager.h"
+#include "popupmenutitle.h"
+#include "panelbutton.h"
 #include "recentapps.h"
 #include "service_mnu.h"
 #include "service_mnu.moc"
-#include "popupmenutitle.h"
-#include "popupposition.h"
-#include "panelbutton.h"
 
 const int idStart = 4242;
 const int kRecentMenusOffset = 1001;
@@ -57,15 +62,17 @@ static RecentlyLaunchedApps s_RecentApps;
 
 PanelServiceMenu::PanelServiceMenu(const QString & label, const QString & relPath, QWidget * parent,
                                    const char * name, bool addmenumode)
-    : KPanelMenu(label, parent, name), relPath_(relPath), clearOnClose_(false),addmenumode_(addmenumode)
+    : KPanelMenu(label, parent, name),
+      relPath_(relPath),
+      clearOnClose_(false),
+      addmenumode_(addmenumode),
+      popupMenu_(0)
 {
-    readConfig();
     subMenus.setAutoDelete(true);
 
     excludeNoDisplay_=true;
 
     connect(KSycoca::self(),  SIGNAL(databaseChanged()),  SLOT(slotClearOnClose()));
-    connect(Kicker::kicker(), SIGNAL(configurationChanged()), this, SLOT(configChanged()) );
     connect(this, SIGNAL(aboutToHide()), this, SLOT(slotClose()));
 }
 
@@ -73,14 +80,6 @@ PanelServiceMenu::~PanelServiceMenu()
 {
 }
 
-void PanelServiceMenu::readConfig()
-{
-    KConfigGroup group( KGlobal::config(), "menus" );
-    merge_              = group.readBoolEntry( "MergeKDEDirs",              TRUE );
-    detailed_           = group.readBoolEntry( "DetailedMenuEntries",       TRUE );
-    detailedNamesFirst_ = group.readBoolEntry( "DetailedEntriesNamesFirst", FALSE );
-}
-
 // create and fill "recent" section at first
 void PanelServiceMenu::createRecentMenuItems()
 {
@@ -90,13 +89,11 @@ void PanelServiceMenu::createRecentMenuItems()
     QStringList RecentApps;
     s_RecentApps.getRecentApps(RecentApps);
 
-    if (RecentApps.count() > 0) {
-        KConfig* config  = KGlobal::config();
-        config->setGroup("menus");
-        bool showTitles = config->readBoolEntry("ShowMenuTitles", true);
-        bool bSeparator = !showTitles;
+    if (RecentApps.count() > 0)
+    {
+        bool bSeparator = KickerSettings::showMenuTitles();
         int nId = idStart + kRecentMenusOffset;
-        int nIndex = showTitles ? 1 : 0;
+        int nIndex = KickerSettings::showMenuTitles() ? 1 : 0;
 
         for (QValueList<QString>::ConstIterator it = RecentApps.fromLast(); ; --it) {
             KService::Ptr s = KService::serviceByDesktopPath(*it);
@@ -104,8 +101,9 @@ void PanelServiceMenu::createRecentMenuItems()
                 s_RecentApps.removeItem(*it);
             else
             {
-                if (!bSeparator) {
-                    bSeparator = true;
+                if (bSeparator)
+                {
+                    bSeparator = false;
                     int id = insertItem(new PopupMenuTitle(s_RecentApps.caption(), font()), idStart + kRecentMenusOffset -1, 0);
                     setItemEnabled( id, false );
                 }
@@ -117,7 +115,7 @@ void PanelServiceMenu::createRecentMenuItems()
                 break;
         }
 
-        if (!showTitles)
+        if (!KickerSettings::showMenuTitles())
         {
             insertSeparator(s_RecentApps.m_nNumMenuItems);
         }
@@ -149,21 +147,22 @@ void PanelServiceMenu::updateRecent()
 
     int nId = idStart + kRecentMenusOffset;
 
-    KConfig* config  = KGlobal::config();
-    config->setGroup("menus");
-    bool showTitles = config->readBoolEntry("ShowMenuTitles", true);
-
     // remove previous items
     if (s_RecentApps.m_nNumMenuItems > 0)
     {
         // -1 --> menu title
-        int i = showTitles ? -1 : 0;
+        int i = KickerSettings::showMenuTitles() ? -1 : 0;
         for (; i < s_RecentApps.m_nNumMenuItems; i++)
         {
             removeItem(nId + i);
             entryMap_.remove(nId + i);
         }
         s_RecentApps.m_nNumMenuItems = 0;
+
+        if (!KickerSettings::showMenuTitles())
+        {
+            removeItemAt(0);
+        }
     }
 
     // insert new items
@@ -172,7 +171,7 @@ void PanelServiceMenu::updateRecent()
 
     if (RecentApps.count() > 0)
     {
-        bool bNeedSeparator = showTitles;
+        bool bNeedSeparator = KickerSettings::showMenuTitles();
         for (QValueList<QString>::ConstIterator it = RecentApps.fromLast(); ; --it)
         {
             KService::Ptr s = KService::serviceByDesktopPath(*it);
@@ -188,7 +187,7 @@ void PanelServiceMenu::updateRecent()
                     int id = insertItem(new PopupMenuTitle(s_RecentApps.caption(), font()), nId - 1, 0);
                     setItemEnabled( id, false );
                 }
-                insertMenuItem(s, nId++, 1);
+                insertMenuItem(s, nId++, KickerSettings::showMenuTitles() ? 1 : 0);
                 s_RecentApps.m_nNumMenuItems++;
             }
 
@@ -196,7 +195,7 @@ void PanelServiceMenu::updateRecent()
                 break;
         }
 
-        if (!showTitles)
+        if (!KickerSettings::showMenuTitles())
         {
             insertSeparator(s_RecentApps.m_nNumMenuItems);
         }
@@ -234,7 +233,6 @@ void PanelServiceMenu::initialize()
 
 void PanelServiceMenu::doInitialize()
 {
-
     // Set the startposition outside the panel, so there is no drag initiated
     // when we use drag and click to select items. A drag is only initiated when
     // you click to open the menu, and then press and drag an item.
@@ -246,7 +244,11 @@ void PanelServiceMenu::doInitialize()
     if (!root || !root->isValid())
         return;
 
-    KServiceGroup::List list = root->entries(true, excludeNoDisplay_, true, detailed_ && !detailedNamesFirst_);
+    KServiceGroup::List list = root->entries(true,
+                                   excludeNoDisplay_,
+                                   true,
+                                   KickerSettings::detailedMenuEntries() &&
+                                   !KickerSettings::detailedEntriesNamesFirst());
 
     if (list.isEmpty()) {
         setItemEnabled(insertItem(i18n("No Entries")), false);
@@ -294,19 +296,47 @@ void PanelServiceMenu::doInitialize()
                 newSubMenu(g->name(), g->relPath(), this, g->name().utf8());
             m->setCaption( groupCaption );
 
-            QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(g->icon(), KIcon::Small,
-                                                                 0, KIcon::DefaultState, 0L, true);
-            QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(g->icon(), KIcon::Small,
-                                                                 0, KIcon::ActiveState, 0L, true);
-            // make sure they are not larger than 20x20
-            if (normal.width() > 20 || normal.height() > 20)
-                normal.convertFromImage(normal.convertToImage().smoothScale(20,20));
-            if (active.width() > 20 || active.height() > 20)
-                active.convertFromImage(active.convertToImage().smoothScale(20,20));
-
             QIconSet iconset;
-            iconset.setPixmap(normal, QIconSet::Small, QIconSet::Normal);
-            iconset.setPixmap(active, QIconSet::Small, QIconSet::Active);
+
+            if (KickerSettings::menuEntryHeight() > 0)
+            {
+                iconset = KGlobal::instance()->iconLoader()->loadIconSet(
+                                                            g->icon(),
+                                                            KIcon::NoGroup,
+                                         KickerSettings::menuEntryHeight());
+            }
+            else if (KickerSettings::menuEntryHeight() == 0)
+            {
+                QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(
+                                                            g->icon(),
+                                                            KIcon::Small,
+                                                            0,
+                                                            KIcon::DefaultState,
+                                                            0L,
+                                                            true);
+
+                QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(
+                                                            g->icon(),
+                                                            KIcon::Small,
+                                                            0,
+                                                            KIcon::ActiveState,
+                                                            0L,
+                                                            true);
+                // make sure they are not larger than 20x20
+                if (normal.width() > 20 || normal.height() > 20)
+                {
+                    normal.convertFromImage(normal.convertToImage()
+                                                  .smoothScale(20,20));
+                }
+                if (active.width() > 20 || active.height() > 20)
+                {
+                    active.convertFromImage(active.convertToImage()
+                                                  .smoothScale(20,20));
+                }
+
+                iconset.setPixmap(normal, QIconSet::Small, QIconSet::Normal);
+                iconset.setPixmap(active, QIconSet::Small, QIconSet::Active);
+            }
 
             int newId = insertItem(iconset, groupCaption, m, id++);
             entryMap_.insert(newId, static_cast<KSycocaEntry*>(g));
@@ -335,7 +365,6 @@ void PanelServiceMenu::configChanged()
 {
     s_RecentApps.m_bNeedToUpdate = false;
     s_RecentApps.configChanged();
-    readConfig();
     deinitialize();
 }
 
@@ -343,10 +372,10 @@ void PanelServiceMenu::insertMenuItem(KService::Ptr & s, int nId, int nIndex/*=
 {
     QString serviceName = s->name();
     // add comment
-    if ( detailed_ ) {
+    if (KickerSettings::detailedMenuEntries()) {
       QString comment = s->genericName();
       if ( !comment.isEmpty() ) {
-        if ( detailedNamesFirst_ )
+        if (KickerSettings::detailedEntriesNamesFirst())
         {
           if (!suppressGenericNames || !suppressGenericNames->contains(s->untranslatedGenericName()))
             serviceName = QString( "%1 (%2)" ).arg( serviceName ).arg( comment );
@@ -374,19 +403,48 @@ void PanelServiceMenu::insertMenuItem(KService::Ptr & s, int nId, int nIndex/*=
     // to accelerators, replace them with two ampersands.
     serviceName.replace("&", "&&");
 
-    QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(s->icon(), KIcon::Small,
-                                                                 0, KIcon::DefaultState, 0L, true);
-    QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(s->icon(), KIcon::Small,
-                                                                 0, KIcon::ActiveState, 0L, true);
-    // make sure they are not larger than 20x20
-    if (normal.width() > 20 || normal.height() > 20)
-        normal.convertFromImage(normal.convertToImage().smoothScale(20,20));
-    if (active.width() > 20 || active.height() > 20)
-        active.convertFromImage(active.convertToImage().smoothScale(20,20));
-
     QIconSet iconset;
-    iconset.setPixmap(normal, QIconSet::Small, QIconSet::Normal);
-    iconset.setPixmap(active, QIconSet::Small, QIconSet::Active);
+
+    if (KickerSettings::menuEntryHeight() > 0)
+    {
+        iconset = KGlobal::instance()->iconLoader()->loadIconSet(
+                                                            s->icon(),
+                                                            KIcon::NoGroup,
+                                          KickerSettings::menuEntryHeight());
+    }
+    else if (KickerSettings::menuEntryHeight() == 0)
+    {
+        QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(
+                                                            s->icon(),
+                                                            KIcon::Small,
+                                                            0,
+                                                            KIcon::DefaultState,
+                                                            0L,
+                                                            true);
+
+        QPixmap active = KGlobal::instance()->iconLoader()->loadIcon(
+                                                            s->icon(),
+                                                            KIcon::Small,
+                                                            0,
+                                                            KIcon::ActiveState,
+                                                            0L,
+                                                            true);
+        // make sure they are not larger than 20x20
+        if (normal.width() > 20 || normal.height() > 20)
+        {
+            normal.convertFromImage(normal.convertToImage()
+                                          .smoothScale(20,20));
+        }
+
+        if (active.width() > 20 || active.height() > 20)
+        {
+            active.convertFromImage(active.convertToImage()
+                                          .smoothScale(20,20));
+        }
+
+        iconset.setPixmap(normal, QIconSet::Small, QIconSet::Normal);
+        iconset.setPixmap(active, QIconSet::Small, QIconSet::Active);
+    }
 
     int newId = insertItem(iconset, serviceName, nId, nIndex);
     entryMap_.insert(newId, static_cast<KSycocaEntry*>(s));
@@ -401,11 +459,11 @@ void PanelServiceMenu::activateParent(const QString &child)
     }
     else
     {
-       PanelPopupButton *kButton = Kicker::kicker()->KButton();
-       if (kButton && (kButton->popup() == this))
+       PanelPopupButton *kButton = MenuManager::the()->findKButtonFor(this);
+       if (kButton)
        {
            adjustSize();
-           popup( popupPosition( kButton->popupDirection(), this, kButton ) );
+           popup(KickerLib::popupPosition(kButton->popupDirection(), this, kButton));
        }
        else
        {
@@ -490,16 +548,141 @@ void PanelServiceMenu::mousePressEvent(QMouseEvent * ev)
 
 void PanelServiceMenu::mouseReleaseEvent(QMouseEvent * ev)
 {
-    if (ev->button()==RightButton)
+    if (ev->button()==RightButton) {
+        int id = idAt( ev->pos() );
+
+        if (id < idStart)
+            return;
+
+        if (!entryMap_.contains(id)) {
+            kdDebug(1210) << "Cannot find service with menu id " << id << endl;
+            return;
+        }
+
+        contextKSycocaEntry_ = entryMap_[id];
+
+        delete popupMenu_;
+        popupMenu_ = new KPopupMenu(this);
+        connect(popupMenu_, SIGNAL(activated(int)), SLOT(slotContextMenu(int)));
+
+        switch (contextKSycocaEntry_->sycocaType()) {
+            case KST_KService:
+                if (kapp->authorize("editable_desktop_icons"))
+		    popupMenu_->insertItem( SmallIconSet("desktop"), i18n("Add Item to Desktop"), AddItemToDesktop );
+                if (kapp->authorizeKAction("kicker_rmb"))
+		    popupMenu_->insertItem( SmallIconSet("kicker"), i18n("Add Item to Main Panel"), AddItemToPanel );
+                if (kapp->authorizeKAction("menuedit"))
+		    popupMenu_->insertItem( SmallIconSet("kmenuedit"), i18n("Edit Item"), EditItem );
+                if (kapp->authorize("run_command"))
+		    popupMenu_->insertItem( SmallIconSet("run"), i18n("Put Into Run Dialog"), PutIntoRunDialog );
+		break;
+
+            case KST_KServiceGroup:
+                if (kapp->authorize("editable_desktop_icons"))
+                    popupMenu_->insertItem( SmallIconSet("desktop"), i18n("Add Menu to Desktop"), AddMenuToDesktop );
+                if (kapp->authorizeKAction("kicker_rmb"))
+                    popupMenu_->insertItem( SmallIconSet("kicker"), i18n("Add Menu to Main Panel"), AddMenuToPanel );
+                if (kapp->authorizeKAction("menuedit"))
+                    popupMenu_->insertItem( SmallIconSet("kmenuedit"), i18n("Edit Menu"), EditMenu );
+                break;
+
+            default:
+               return;
+               break;
+        }
+
+        popupMenu_->popup( this->mapToGlobal( ev->pos() ) );
+
       return;
+    }
+
+    delete popupMenu_;
+    popupMenu_ = 0;
 
     KPanelMenu::mouseReleaseEvent(ev);
 }
 
+void PanelServiceMenu::slotContextMenu(int selected)
+{
+    KProcess *proc;
+    KService::Ptr service;
+    KServiceGroup::Ptr g;
+    QByteArray ba;
+    QDataStream ds(ba, IO_WriteOnly);
+
+    KURL src,dest;
+    KIO::CopyJob *job;
+    KDesktopFile *df;
+
+    switch (selected) {
+        case AddItemToDesktop:
+	    service = static_cast<KService *>(contextKSycocaEntry_);
+
+	    src.setPath( KGlobal::dirs()->findResource( "apps", service->desktopEntryPath() ) );
+	    dest.setPath( KGlobalSettings::desktopPath() );
+	    dest.setFileName( src.fileName() );
+
+            job = KIO::copyAs( src, dest );
+            job->setDefaultPermissions( true );
+	    break;
+
+	case AddItemToPanel:
+	    service = static_cast<KService *>(contextKSycocaEntry_);
+	    kapp->dcopClient()->send("kicker", "Panel", "addServiceButton(QString)", service->desktopEntryPath());
+	    break;
+
+	case EditItem:
+            proc = new KProcess(this);
+            *proc << KStandardDirs::findExe(QString::fromLatin1("kmenuedit"));
+            *proc << "/"+relPath_ << static_cast<KService *>(contextKSycocaEntry_)->menuId();
+            proc->start();
+	    break;
+
+	case PutIntoRunDialog:
+	    service = static_cast<KService *>(contextKSycocaEntry_);
+	    kapp->dcopClient()->send("kdesktop", "default", "popupExecuteCommand(QString)", service->exec());
+	    break;
+
+	case AddMenuToDesktop:
+	    g = static_cast<KServiceGroup *>(contextKSycocaEntry_);
+            dest.setPath( KGlobalSettings::desktopPath() );
+	    dest.setFileName( g->caption() );
+
+	    df = new KDesktopFile( dest.path() );
+            df->writeEntry( "Icon", g->icon() );
+            df->writePathEntry( "URL", "programs:/"+g->name() );
+	    df->writeEntry( "Name", g->caption() );
+	    df->writeEntry( "Type", "Link" );
+            df->sync();
+	    delete df;
+
+	    break;
+
+	case AddMenuToPanel:
+	    g = static_cast<KServiceGroup *>(contextKSycocaEntry_);
+            ds << "foo" << g->relPath();
+	    kapp->dcopClient()->send("kicker", "Panel", "addServiceMenuButton(QString,QString)", ba);
+	    break;
+
+        case EditMenu:
+            proc = new KProcess(this);
+            *proc << KStandardDirs::findExe(QString::fromLatin1("kmenuedit"));
+            *proc << "/"+static_cast<KServiceGroup *>(contextKSycocaEntry_)->relPath();
+            proc->start();
+	    break;
+
+	default:
+	    break;
+	}
+}
+
 void PanelServiceMenu::mouseMoveEvent(QMouseEvent * ev)
 {
     KPanelMenu::mouseMoveEvent(ev);
 
+    if (Kicker::the()->isImmutable())
+        return;
+
     if ( (ev->state() & LeftButton ) != LeftButton )
         return;
 
@@ -520,41 +703,45 @@ void PanelServiceMenu::mouseMoveEvent(QMouseEvent * ev)
 
     KSycocaEntry * e = entryMap_[id];
 
-    KService::Ptr service = static_cast<KService *>(e);
-
-    QString filePath;
-
     QPixmap icon;
+    KURL url;
 
     switch (e->sycocaType()) {
 
         case KST_KService:
-            icon     = static_cast<KService *>(e)->pixmap(KIcon::Small);
-            filePath = static_cast<KService *>(e)->desktopEntryPath();
+        {
+            icon = static_cast<KService *>(e)->pixmap(KIcon::Small);
+            QString filePath = static_cast<KService *>(e)->desktopEntryPath();
+            if (filePath[0] != '/')
+            {
+                filePath = locate("apps", filePath);
+            }
+            url.setPath(filePath);
             break;
+        }
 
         case KST_KServiceGroup:
+        {
             icon = KGlobal::iconLoader()
                    ->loadIcon(static_cast<KServiceGroup *>(e)->icon(), KIcon::Small);
-            filePath = static_cast<KServiceGroup *>(e)->relPath();
+            url = "programs:/" + static_cast<KServiceGroup *>(e)->relPath();
             break;
+        }
 
         default:
+        {
             return;
             break;
+        }
     }
 
     // If the path to the desktop file is relative, try to get the full
     // path from KStdDirs.
 
-    QString path = (filePath[0] == '/') ? filePath : locate("apps", filePath);
-    KURL url;
-    url.setPath(path);
     KURLDrag *d = new KURLDrag(KURL::List(url), this);
-
+    connect(d, SIGNAL(destroyed()), this, SLOT(slotDragObjectDestroyed()));
     d->setPixmap(icon);
     d->dragCopy();
-    //close();
 
     // Set the startposition outside the panel, so there is no drag initiated
     // when we use drag and click to select items. A drag is only initiated when
@@ -562,6 +749,43 @@ void PanelServiceMenu::mouseMoveEvent(QMouseEvent * ev)
     startPos_ = QPoint(-1,-1);
 }
 
+void PanelServiceMenu::dragEnterEvent(QDragEnterEvent *event)
+{
+    // Set the DragObject's target to this widget. This is needed because the
+    // widget doesn't accept drops, but we want to determine if the drag object
+    // is dropped on it. This avoids closing on accidental drags. If this
+    // widget accepts drops in the future, these lines can be removed.
+    if (event->source() == this)
+    {
+        KURLDrag::setTarget(this);
+    }
+    event->ignore();
+}
+
+void PanelServiceMenu::dragLeaveEvent(QDragLeaveEvent */*event*/)
+{
+    // see PanelServiceMenu::dragEnterEvent why this is nescessary
+    if (!frameGeometry().contains(QCursor::pos()))
+    {
+        KURLDrag::setTarget(0);
+    }
+}
+
+void PanelServiceMenu::slotDragObjectDestroyed()
+{
+    if (KURLDrag::target() != this)
+    {
+        // we need to re-enter the event loop before calling close() here
+        // this gets called _before_ the drag object is destroyed, so we are
+        // still in its event loop. closing the menu before that event loop is
+        // exited may result in getting hung up in it which in turn prevents
+        // the execution of any code after the original exec() statement
+        // though the panels themselves continue on otherwise normally
+        // (we just have some sort of nested event loop)
+        QTimer::singleShot(0, this, SLOT(close()));
+    }
+}
+
 PanelServiceMenu *PanelServiceMenu::newSubMenu(const QString & label, const QString & relPath,
                                                QWidget * parent, const char * name)
 {
@@ -586,6 +810,9 @@ void PanelServiceMenu::slotClose()
         clearOnClose_ = false;
         slotClear();
     }
+    
+    delete popupMenu_;
+    popupMenu_ = 0;
 }
 
 void PanelServiceMenu::slotClear()
