@@ -626,7 +626,7 @@ void MANProtocol::stat( const KURL& url)
 extern "C"
 {
 
-    int kdemain( int argc, char **argv ) {
+    int KDE_EXPORT kdemain( int argc, char **argv ) {
 
         KInstance instance("kio_man");
 
