@@ -66,11 +66,12 @@ static const KCmdLineOptions options[] =
    KCmdLineLastOption
 };
 
-extern "C" int kdemain( int argc, char **argv )
+extern "C" KDE_EXPORT int kdemain( int argc, char **argv )
 {
   KCmdLineArgs::init(argc, argv, appName, programName, description, version, false);
 
   KCmdLineArgs::addCmdLineOptions( options );
+  KCmdLineArgs::addTempFileOption();
 
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
@@ -272,16 +273,19 @@ static QCString konqyToReuse( const QString& url, const QString& mimetype, const
     return ret;
 }
 
-bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & mimetype)
+bool clientApp::createNewWindow(const KURL & url, bool newTab, bool tempFile, const QString & mimetype)
 {
     kdDebug( 1202 ) << "clientApp::createNewWindow " << url.url() << " mimetype=" << mimetype << endl;
-
     // check if user wants to use external browser
+    // ###### this option seems to have no GUI and to be redundant with BrowserApplication now.
+    // ###### KDE4: remove
     KConfig config( QString::fromLatin1("kfmclientrc"));
     config.setGroup( QString::fromLatin1("Settings"));
     QString strBrowser = config.readPathEntry("ExternalBrowser");
     if (!strBrowser.isEmpty())
     {
+        if ( tempFile )
+            kdWarning() << "kfmclient used with --tempfile but is passing to an external browser! Tempfile will never be deleted" << endl;
         KProcess proc;
         proc << strBrowser << url.url();
         proc.start( KProcess::DontCare );
@@ -295,8 +299,8 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & m
         {
             clientApp app;
             KStartupInfo::appStarted();
-            
-            KRun * run = new KRun( url );
+
+            KRun * run = new KRun( url ); // TODO pass tempFile [needs support in the KRun ctor]
             QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
             QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
             app.exec();
@@ -315,7 +319,7 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & m
             "windowCanBeUsedForTab()", data, foundApp, foundObj, false, 3000 ) )
         {
             DCOPRef ref( foundApp, foundObj );
-            DCOPReply reply = ref.call( "newTab", url.url() );
+            DCOPReply reply = ref.call( "newTab", url.url(), tempFile );
             if ( reply.isValid() ) {
                 KStartupInfo::appStarted();
                 return true;
@@ -328,7 +332,7 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & m
     {
         kdDebug( 1202 ) << "clientApp::createNewWindow using existing konqueror" << endl;
         KonquerorIface_stub konqy( appId, "KonquerorIface" );
-        konqy.createNewWindowASN( url.url(), mimetype, startup_id_str );
+        konqy.createNewWindowASN( url.url(), mimetype, startup_id_str, tempFile );
         KStartupInfoId id;
         id.initId( startup_id_str );
         KStartupInfoData data;
@@ -356,10 +360,12 @@ bool clientApp::createNewWindow(const KURL & url, bool newTab, const QString & m
             id.initId( startup_id_str );
             id.setupStartupEnv();
             KProcess proc;
-            if ( mimetype.isEmpty() )
-                proc << QString::fromLatin1("kshell") << QString::fromLatin1("konqueror") << url.url();
-            else
-                proc << QString::fromLatin1("kshell") << QString::fromLatin1("konqueror") << QString::fromLatin1("-mimetype") << mimetype << url.url();
+            proc << "kshell" << "konqueror";
+            if ( !mimetype.isEmpty() )
+                proc << "-mimetype" << mimetype;
+            if ( tempFile )
+                proc << "-tempfile";
+            proc << url.url();
             proc.start( KProcess::DontCare );
             KStartupInfo::resetStartupEnv();
             kdDebug( 1202 ) << "clientApp::createNewWindow KProcess started" << endl;
@@ -454,19 +460,20 @@ bool clientApp::doIt()
 	KApplication::dcopClient()->attach();
     }
     checkArgumentCount(argc, 1, 3);
+    bool tempFile = KCmdLineArgs::isTempFileSet();
     if ( argc == 1 )
     {
       KURL url;
       url.setPath(QDir::homeDirPath());
-      return createNewWindow( url, command == "newTab" );
+      return createNewWindow( url, command == "newTab", tempFile );
     }
     if ( argc == 2 )
     {
-      return createNewWindow( args->url(1), command == "newTab" );
+      return createNewWindow( args->url(1), command == "newTab", tempFile );
     }
     if ( argc == 3 )
     {
-      return createNewWindow( args->url(1), command == "newTab", QString::fromLatin1(args->arg(2)) );
+      return createNewWindow( args->url(1), command == "newTab", tempFile, QString::fromLatin1(args->arg(2)) );
     }
   }
   else if ( command == "openProfile" )
@@ -505,7 +512,6 @@ bool clientApp::doIt()
     }
     else if ( argc == 2 )
     {
-      KFileOpenWithHandler fowh;
       KRun * run = new KRun( args->url(1) );
       QObject::connect( run, SIGNAL( finished() ), &app, SLOT( delayedQuit() ));
       QObject::connect( run, SIGNAL( error() ), &app, SLOT( delayedQuit() ));
@@ -516,9 +522,9 @@ bool clientApp::doIt()
     {
       KURL::List urls;
       urls.append( args->url(1) );
-      KService::Ptr serv = (*KTrader::self()->query( QString::fromLocal8Bit(args->arg(2)) ).begin());
-      if (!serv) return 1;
-      KFileOpenWithHandler fowh;
+      const KTrader::OfferList offers = KTrader::self()->query( QString::fromLocal8Bit( args->arg( 2 ) ), QString::fromLatin1( "Application" ), QString::null, QString::null );
+      if (offers.isEmpty()) return 1;
+      KService::Ptr serv = offers.first();
       return KRun::run( *serv, urls );
     }
   }
