@@ -39,22 +39,34 @@
 #include "konq_viewmgr.h"
 #include "konq_misc.h"
 
+#include <kaccelmanager.h>
 #include <konq_pixmapprovider.h>
 #include <kstdaccel.h>
 #include <qtabbar.h>
+#include <qwhatsthis.h>
 #include <qstyle.h>
 
+#define DUPLICATE_ID 3
+#define RELOAD_ID 4
 #define BREAKOFF_ID 5
 #define CLOSETAB_ID 6
-#define RELOAD_ALL_ID 7
-#define CLOSE_OTHER_ID 8
+#define OTHERTABS_ID 7
 
 //###################################################################
 
 KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentContainer,
                              KonqViewManager* viewManager, const char * name)
-  : KTabWidget(parent, name), m_rightWidget(0), m_leftWidget(0), m_alwaysTabBar(false)
+  : KTabWidget(parent, name), m_rightWidget(0), m_leftWidget(0), m_alwaysTabBar(false),
+    m_closeOtherTabsId(0)
 {
+  KAcceleratorManager::setNoAccel(this);
+
+  QWhatsThis::add( tabBar(), i18n( "This bar contains list of currently open tabs. Click on a tab to make it "
+			  "active. The option to show a close button instead of website icon in the left "
+			  "corner of the tab is configurable. You can also use keyboard shortcuts to "
+			  "navigate through tabs. The text on the tab is the title of the website "
+			  "currently open in it, put your mouse over the tab too see the full title in "
+			  "case it was truncated to fit the tab size." ) );
   //kdDebug(1202) << "KonqFrameTabs::KonqFrameTabs()" << endl;
 
   m_pParentContainer = parentContainer;
@@ -72,59 +84,45 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
                             m_pViewManager->mainWindow(),
                             SLOT( slotAddTab() ),
                             m_pViewManager->mainWindow()->action("newtab")->shortcut() );
-  m_pPopupMenu->insertItem( SmallIcon( "tab_duplicate" ),
+  m_pPopupMenu->insertItem( SmallIconSet( "tab_duplicate" ),
                             i18n("&Duplicate Tab"),
                             m_pViewManager->mainWindow(),
                             SLOT( slotDuplicateTabPopup() ),
-                            m_pViewManager->mainWindow()->action("duplicatecurrenttab")->shortcut() );
+                            m_pViewManager->mainWindow()->action("duplicatecurrenttab")->shortcut(), 
+                            DUPLICATE_ID );
+  m_pPopupMenu->insertItem( SmallIconSet( "reload" ),
+                            i18n( "&Reload Tab" ),
+                            m_pViewManager->mainWindow(),
+                            SLOT( slotReloadPopup() ),
+                            m_pViewManager->mainWindow()->action("reload")->shortcut(), RELOAD_ID );
   m_pPopupMenu->insertSeparator();
-  m_pPopupMenu->insertItem( SmallIcon( "tab_breakoff" ),
+  m_pSubPopupMenuTab = new QPopupMenu( this );
+  m_pPopupMenu->insertItem( i18n("Other Tabs" ), m_pSubPopupMenuTab, OTHERTABS_ID );
+  connect( m_pSubPopupMenuTab, SIGNAL( activated ( int ) ),
+           this, SLOT( slotSubPopupMenuTabActivated( int ) ) );
+  m_pPopupMenu->insertSeparator();
+  m_pPopupMenu->insertItem( SmallIconSet( "tab_breakoff" ),
                             i18n("D&etach Tab"),
                             m_pViewManager->mainWindow(),
                             SLOT( slotBreakOffTabPopup() ),
                             m_pViewManager->mainWindow()->action("breakoffcurrenttab")->shortcut(),
                             BREAKOFF_ID );
-  m_pPopupMenu->insertItem( SmallIcon( "tab_remove" ),
-			    i18n("&Close Tab"),
+  m_pPopupMenu->insertSeparator();
+  m_pPopupMenu->insertItem( SmallIconSet( "tab_remove" ),
+                            i18n("&Close Tab"),
                             m_pViewManager->mainWindow(),
                             SLOT( slotRemoveTabPopup() ),
                             m_pViewManager->mainWindow()->action("removecurrenttab")->shortcut(),
                             CLOSETAB_ID );
-  m_pPopupMenu->insertSeparator();
-  m_pPopupMenu->insertItem( SmallIcon( "reload" ),
-                            i18n( "&Reload" ),
-                            m_pViewManager->mainWindow(),
-                            SLOT( slotReloadPopup() ),
-                            m_pViewManager->mainWindow()->action("reload")->shortcut() );
-  m_pPopupMenu->insertItem( SmallIcon( "reload_all_tabs" ),
-                            i18n( "&Reload All Tabs" ),
-                            m_pViewManager->mainWindow(),
-                            SLOT( slotReloadAllTabs() ),
-                            m_pViewManager->mainWindow()->action("reload_all_tabs")->shortcut(),
-                            RELOAD_ALL_ID );
-  m_pPopupMenu->insertSeparator();
-  m_pSubPopupMenuTab = new QPopupMenu( this );
-  m_pPopupMenu->insertItem( i18n("Switch to Tab" ), m_pSubPopupMenuTab );
-  connect( m_pSubPopupMenuTab, SIGNAL( activated ( int ) ),
-           this, SLOT( slotSubPopupMenuTabActivated( int ) ) );
-  m_pPopupMenu->insertSeparator();
-  m_pPopupMenu->insertItem( SmallIcon( "tab_remove" ),
-                            i18n( "Close &Other Tabs" ),
-                            m_pViewManager->mainWindow(),
-                            SLOT( slotRemoveOtherTabsPopup() ),
-                            m_pViewManager->mainWindow()->action("removeothertabs")->shortcut(),
-                            CLOSE_OTHER_ID );
   connect( this, SIGNAL( contextMenu( QWidget *, const QPoint & ) ),
            SLOT(slotContextMenu( QWidget *, const QPoint & ) ) );
+  connect( this, SIGNAL( contextMenu( const QPoint & ) ),
+           SLOT(slotContextMenu( const QPoint & ) ) );
 
   KConfig *config = KGlobal::config();
   KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
 
-  m_maxLength = config->readNumEntry("MaximumTabLength", 30);
-  m_minLength = config->readNumEntry("MinimumTabLength", 3);
-  if (m_minLength>m_maxLength)
-    m_maxLength=m_minLength;
-  m_CurrentMaxLength = m_maxLength;
+  m_MouseMiddleClickClosesTab = config->readBoolEntry( "MouseMiddleClickClosesTab", false );
 
   m_permanentCloseButtons = config->readBoolEntry( "PermanentCloseButton", false );
   if (m_permanentCloseButtons) {
@@ -153,12 +151,13 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
     m_rightWidget = new QToolButton( this );
     connect( m_rightWidget, SIGNAL( clicked() ),
              m_pViewManager->mainWindow(), SLOT( slotRemoveTab() ) );
-    m_rightWidget->setIconSet( SmallIcon( "tab_remove" ) );
+    m_rightWidget->setIconSet( SmallIconSet( "tab_remove" ) );
     m_rightWidget->adjustSize();
     QToolTip::add(m_rightWidget, i18n("Close the current tab"));
     setCornerWidget( m_rightWidget, TopRight );
   }
 
+  setAutomaticResizeTabs( true );
   setTabReorderingEnabled( true );
   connect( this, SIGNAL( movedTab( int, int ) ),
            SLOT( slotMovedTab( int, int ) ) );
@@ -251,99 +250,10 @@ void KonqFrameTabs::reparentFrame( QWidget* parent, const QPoint & p, bool showI
   QWidget::reparent( parent, p, showIt );
 }
 
-uint KonqFrameTabs::tabBarWidthForMaxChars( uint maxLength )
-{
-  int hframe, overlap;
-  hframe  = tabBar()->style().pixelMetric( QStyle::PM_TabBarTabHSpace, this );
-  overlap = tabBar()->style().pixelMetric( QStyle::PM_TabBarTabOverlap, this );
-
-  QFontMetrics fm = tabBar()->fontMetrics();
-  int x = 0;
-  for( int i=0; i < count(); ++i ) {
-    QString newTitle;
-    KonqFrame* konqframe = dynamic_cast<KonqFrame*>( page(i) );
-    if ( konqframe )
-      newTitle = konqframe->title();
-    else {
-      KonqView* konqview= static_cast<KonqFrameContainer*>( page(i) )->activeChildView();
-      if ( konqview )
-        newTitle = konqview->frame()->title();
-    }
-
-    newTitle = KStringHandler::rsqueeze( newTitle, maxLength ).leftJustify( m_minLength, ' ' );
-
-    QTab* tab = tabBar()->tabAt( i );
-    int lw = fm.width( newTitle );
-    int iw = 0;
-    if ( tab->iconSet() )
-      iw = tab->iconSet()->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 4;
-    x += ( tabBar()->style().sizeFromContents( QStyle::CT_TabBarTab, this,
-                   QSize( QMAX( lw + hframe + iw, QApplication::globalStrut().width() ), 0 ),
-                   QStyleOption( tab ) ) ).width();
-  }
-  return x;
-}
-
 void KonqFrameTabs::setTitle( const QString &title , QWidget* sender)
 {
   // kdDebug(1202) << "KonqFrameTabs::setTitle( " << title << " , " << sender << " )" << endl;
-  removeTabToolTip( sender );
-
-  uint lcw=0, rcw=0;
-
-  int tabBarHeight = tabBar()->sizeHint().height();
-  if ( cornerWidget( TopLeft ) && cornerWidget( TopLeft )->isVisible() )
-    lcw = QMAX( cornerWidget( TopLeft )->width(), tabBarHeight );
-  if ( cornerWidget( TopRight ) && cornerWidget( TopRight )->isVisible() )
-    rcw = QMAX( cornerWidget( TopRight )->width(), tabBarHeight );
-
-  uint maxTabBarWidth = width() - lcw - rcw;
-  // kdDebug(1202) << "maxTabBarWidth=" << maxTabBarWidth << endl;
-
-  uint newMaxLength=m_maxLength;
-  for ( ; newMaxLength > m_minLength; newMaxLength-- ) {
-    // kdDebug(1202) << "tabBarWidthForMaxChars(" << newMaxLength
-    //               << ")=" << tabBarWidthForMaxChars( newMaxLength ) << endl;
-    if ( tabBarWidthForMaxChars( newMaxLength ) < maxTabBarWidth )
-      break;
-  }
-  // kdDebug(1202) << "newMaxLength=" << newMaxLength << endl;
-
-  QString newTitle = title;
-  if ( newTitle.length() > newMaxLength )
-      setTabToolTip( sender, newTitle );
-
-  newTitle = KStringHandler::rsqueeze( newTitle, newMaxLength ).leftJustify( m_minLength, ' ' );
-
-  newTitle.replace( '&', "&&" );
-  if ( tabLabel( sender ) != newTitle )
-    changeTab( sender, newTitle );
-
-  if( newMaxLength != m_CurrentMaxLength )
-    {
-      for( int i = 0; i < count(); ++i)
-        {
-          KonqFrame* konqframe = dynamic_cast<KonqFrame*>( page(i) );
-          if ( konqframe )
-            newTitle = konqframe->title();
-          else {
-            KonqView* konqview= static_cast<KonqFrameContainer*>( page(i) )->activeChildView();
-            if ( konqview )
-              newTitle = konqview->frame()->title();
-          }
-
-          removeTabToolTip( page( i ) );
-          if ( newTitle.length() > newMaxLength )
-              setTabToolTip( page( i ), newTitle );
-
-          newTitle = KStringHandler::rsqueeze( newTitle, newMaxLength ).leftJustify( m_minLength, ' ' );
-
-          newTitle.replace( '&', "&&" );
-          if ( newTitle != tabLabel( page( i ) ) )
-            changeTab( page( i ), newTitle );
-        }
-      m_CurrentMaxLength = newMaxLength;
-    }
+  setTabLabel( sender,title );
 }
 
 void KonqFrameTabs::setTabIcon( const QString &url, QWidget* sender )
@@ -353,7 +263,7 @@ void KonqFrameTabs::setTabIcon( const QString &url, QWidget* sender )
   if (m_permanentCloseButtons)
     iconSet =  SmallIcon( "fileclose" );
   else
-    iconSet =  QIconSet( KonqPixmapProvider::self()->pixmapFor( url ) );
+    iconSet =  SmallIconSet( KonqPixmapProvider::self()->iconNameFor( url ) );
   if (tabIconSet( sender ).pixmap().serialNumber() != iconSet.pixmap().serialNumber())
     setTabIconSet( sender, iconSet );
 }
@@ -387,7 +297,7 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
         activeChildView->setTabIcon( activeChildView->url().url() );
       }
       if (showTabBar)
-          this->showTabBar();
+          setTabBarHidden(false);
       else if ( count() == 1 )
           this->hideTabBar();//the first frame inserted (initialization)
     }
@@ -450,15 +360,37 @@ void KonqFrameTabs::slotMovedTab( int from, int to )
   }
 }
 
+void KonqFrameTabs::slotContextMenu( const QPoint &p )
+{
+  refreshSubPopupMenuTab();
+  
+  m_pPopupMenu->setItemEnabled( RELOAD_ID, false );
+  m_pPopupMenu->setItemEnabled( DUPLICATE_ID, false );
+  m_pPopupMenu->setItemEnabled( BREAKOFF_ID, false );
+  m_pPopupMenu->setItemEnabled( CLOSETAB_ID, false );
+  m_pPopupMenu->setItemEnabled( OTHERTABS_ID, true );
+  m_pSubPopupMenuTab->setItemEnabled( m_closeOtherTabsId, false );
+ 
+  m_pPopupMenu->exec( p );
+}
+
 void KonqFrameTabs::slotContextMenu( QWidget *w, const QPoint &p )
 {
-  m_pPopupMenu->setItemEnabled( BREAKOFF_ID, m_pChildFrameList->count()>1 );
-  m_pPopupMenu->setItemEnabled( CLOSETAB_ID, m_pChildFrameList->count()>1 );
-  m_pPopupMenu->setItemEnabled( RELOAD_ALL_ID, m_pChildFrameList->count()>1 );
-  m_pPopupMenu->setItemEnabled( CLOSE_OTHER_ID, m_pChildFrameList->count()>1 );
-  // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
-  m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
   refreshSubPopupMenuTab();
+  
+  uint tabCount = m_pChildFrameList->count();
+  m_pPopupMenu->setItemEnabled( RELOAD_ID, true );
+  m_pPopupMenu->setItemEnabled( DUPLICATE_ID, true );
+  m_pPopupMenu->setItemEnabled( BREAKOFF_ID, tabCount>1 );
+  m_pPopupMenu->setItemEnabled( CLOSETAB_ID, tabCount>1 );
+  m_pPopupMenu->setItemEnabled( OTHERTABS_ID, tabCount>1 );
+  m_pSubPopupMenuTab->setItemEnabled( m_closeOtherTabsId, true );
+
+  // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a
+  // class hierarchy and it could crash one day, but I haven't checked
+  // setWorkingTab so I don't know if it can handle nulls.
+
+  m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
   m_pPopupMenu->exec( p );
 }
 
@@ -466,6 +398,12 @@ void KonqFrameTabs::refreshSubPopupMenuTab()
 {
     m_pSubPopupMenuTab->clear();
     int i=0;
+    m_pSubPopupMenuTab->insertItem( SmallIcon( "reload_all_tabs" ),
+                                    i18n( "&Reload All Tabs" ),
+                                    m_pViewManager->mainWindow(),
+                                    SLOT( slotReloadAllTabs() ),
+                                    m_pViewManager->mainWindow()->action("reload_all_tabs")->shortcut() );
+    m_pSubPopupMenuTab->insertSeparator();
     for (KonqFrameBase* it = m_pChildFrameList->first(); it; it = m_pChildFrameList->next())
     {
         KonqFrame* frame = dynamic_cast<KonqFrame *>(it);
@@ -474,12 +412,19 @@ void KonqFrameTabs::refreshSubPopupMenuTab()
             QString title = frame->title().stripWhiteSpace();
             if ( title.isEmpty() )
                 title = frame->activeChildView()->url().url();
+            title = KStringHandler::csqueeze( title, 50 );
             m_pSubPopupMenuTab->insertItem( QIconSet( KonqPixmapProvider::self()->pixmapFor( frame->activeChildView()->url().url() ) ), title, i );
 
         }
         i++;
     }
-
+    m_pSubPopupMenuTab->insertSeparator();
+    m_closeOtherTabsId =
+      m_pSubPopupMenuTab->insertItem( SmallIconSet( "tab_remove" ),
+				      i18n( "Close &Other Tabs" ),
+				      m_pViewManager->mainWindow(),
+				      SLOT( slotRemoveOtherTabsPopup() ),
+				      m_pViewManager->mainWindow()->action("removeothertabs")->shortcut() );
 }
 
 void KonqFrameTabs::slotCloseRequest( QWidget *w )
@@ -511,6 +456,14 @@ void KonqFrameTabs::slotMouseMiddleClick()
 
 void KonqFrameTabs::slotMouseMiddleClick( QWidget *w )
 {
+  if ( m_MouseMiddleClickClosesTab ) {
+    if ( m_pChildFrameList->count() > 1 ) {
+      // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
+      m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
+      emit ( removeTabPopup() );
+    }
+  }
+  else {
   QApplication::clipboard()->setSelectionMode( QClipboard::Selection );
   KURL filteredURL ( KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text() ) );
   if ( !filteredURL.isEmpty() ) {
@@ -519,21 +472,7 @@ void KonqFrameTabs::slotMouseMiddleClick( QWidget *w )
       m_pViewManager->mainWindow()->openURL( frame->activeChildView(), filteredURL );
     }
   }
-}
-
-void KonqFrameTabs::resizeEvent( QResizeEvent *e )
-{
-    KTabWidget::resizeEvent( e );
-    if ( count() ) {
-      KonqFrame* konqframe = dynamic_cast<KonqFrame*>( page(0) );
-      if ( konqframe )
-        setTitle( konqframe->title(), page( 0 ) );
-      else {
-        KonqView* konqview= static_cast<KonqFrameContainer*>( page(0) )->activeChildView();
-        if ( konqview )
-          setTitle( konqview->frame()->title(), page( 0 ) );
-      }
-    }
+  }
 }
 
 void KonqFrameTabs::slotTestCanDecode(const QDragMoveEvent *e, bool &accept /* result */)
@@ -581,25 +520,12 @@ void KonqFrameTabs::slotInitiateDrag( QWidget *w )
 void KonqFrameTabs::hideTabBar()
 {
   if ( !m_alwaysTabBar ) {
-    if (m_leftWidget)
-      m_leftWidget->hide();
-    if (m_rightWidget)
-      m_rightWidget->hide();
-    tabBar()->hide();
+    setTabBarHidden(true);
   }
   m_pPopupMenu->setItemEnabled( BREAKOFF_ID, false );
   m_pPopupMenu->setItemEnabled( CLOSETAB_ID, false );
 }
 
-void KonqFrameTabs::showTabBar()
-{
-  tabBar()->show();
-  if (m_leftWidget)
-    m_leftWidget->show();
-  if (m_rightWidget)
-    m_rightWidget->show();
-}
-
 void KonqFrameTabs::setAlwaysTabbedMode( bool enable )
 {
   bool update = ( enable != m_alwaysTabBar );
@@ -607,7 +533,7 @@ void KonqFrameTabs::setAlwaysTabbedMode( bool enable )
   m_alwaysTabBar = enable;
   if ( update ) {
     if ( m_alwaysTabBar )
-      showTabBar();
+      setTabBarHidden(false);
     else
       hideTabBar();
   }
