@@ -54,6 +54,7 @@ struct HistoryEntry
   QString postContentType;
   bool doPost;
   QString pageReferrer;
+  KonqMainWindow::PageSecurity pageSecurity;
 };
 
 /* This class represents a child of the main view. The main view maintains
@@ -94,10 +95,12 @@ public:
    * @param url the URL to open
    * @param locationBarURL the URL to set in the location bar (see @ref setLocationBarURL)
    * @param nameFilter e.g. *.cpp
+   * @param tempFile whether to delete the file after use
    */
   void openURL( const KURL &url,
                 const QString & locationBarURL,
-                const QString &nameFilter = QString::null);
+                const QString &nameFilter = QString::null,
+                bool tempFile = false );
 
   /**
    * Change the type of view (i.e. loads a new konqueror view)
@@ -129,6 +132,11 @@ public:
    */
   bool canGoForward()const { return m_lstHistory.at() != ((int)m_lstHistory.count())-1; }
 
+  /**
+   * @return the position in the history
+   */
+  int historyPos() const { return m_lstHistory.at(); }
+
   uint historyLength() { return m_lstHistory.count(); }
 
   /**
@@ -136,11 +144,23 @@ public:
    */
   void go( int steps );
 
+  /**
+   * Helper function for go() and KonqViewManager
+   */
+  void restoreHistory();
+
+  void setHistoryPos(int newPos) { m_lstHistory.at( newPos ); }
+
   /**
    * @return the history of this view
    */
   const QPtrList<HistoryEntry> & history() { return m_lstHistory; }
 
+  /**
+   * @return the HistoryEntry at postion @p pos
+   */
+  const HistoryEntry* historyAt(const int pos);
+
   /**
    * Creates a deep copy of the @p other view's history buffers.
    */
@@ -181,6 +201,11 @@ public:
    */
   void setTypedURL( const QString & u ) { m_sTypedURL = u; }
 
+  /**
+   * Return the security state of page in view
+   */
+  KonqMainWindow::PageSecurity pageSecurity() const { return m_pageSecurity; }
+  
   /**
    * @return the part embedded into this view
    */
@@ -289,6 +314,9 @@ public:
   // Returns false in case the reload must be cancelled.
   bool prepareReload( KParts::URLArgs& args );
 
+  // overload for the QString version
+  void setLocationBarURL( const KURL& locationBarURL );
+
   static QStringList childFrameNames( KParts::ReadOnlyPart *part );
 
   static KParts::BrowserHostExtension *hostExtension( KParts::ReadOnlyPart *part, const QString &name );
@@ -326,6 +354,8 @@ public slots:
 
   void setCaption( const QString & caption );
 
+  void setPageSecurity( int );
+  
   // connected to the KROP's KIO::Job
   // but also to KonqRun's job
   void slotInfoMessage( KIO::Job *, const QString &msg );
@@ -379,6 +409,8 @@ private:
 
   void setServiceTypeInExtension();
 
+  void finishedWithCurrentURL();
+
   virtual bool eventFilter( QObject *obj, QEvent *e );
 
 ////////////////// private members ///////////////
@@ -387,6 +419,7 @@ private:
 
   QString m_sLocationBarURL;
   QString m_sTypedURL;
+  KonqMainWindow::PageSecurity m_pageSecurity;
 
   /**
    * The full history (back + current + forward)
@@ -426,15 +459,16 @@ private:
   uint m_bPendingRedirection:1;
   uint m_bBuiltinView:1;
   uint m_bURLDropHandling:1;
+  uint m_bBackRightClick:1;
+  uint m_bHierarchicalView:1;
   KTrader::OfferList m_partServiceOffers;
   KTrader::OfferList m_appServiceOffers;
   KService::Ptr m_service;
   QString m_serviceType;
   QString m_caption;
+  QString m_tempFile;
   KonqViewIface * m_dcopObject;
   KonqBrowserInterface *m_browserIface;
-  bool m_bBackRightClick;
-  bool m_bHierarchicalView;
   int m_randID;
 };
 
