@@ -12,21 +12,24 @@ License. See the file "COPYING" for the exact licensing terms.
 #ifndef KWIN_CLIENT_H
 #define KWIN_CLIENT_H
 
-#include "utils.h"
-#include "options.h"
-#include "workspace.h"
-#include "kdecoration.h"
 #include <qframe.h>
 #include <qvbox.h>
 #include <qpixmap.h>
 #include <netwm.h>
 #include <kdebug.h>
 #include <assert.h>
+#include <kshortcut.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <fixx11h.h>
 
+#include "utils.h"
+#include "options.h"
+#include "workspace.h"
+#include "kdecoration.h"
+#include "rules.h"
+
 class QTimer;
 class KProcess;
 class KStartupInfoData;
@@ -63,12 +66,15 @@ class Client : public QObject, public KDecorationDefines
         Client* findModal();
         const Group* group() const;
         Group* group();
+        void checkGroup( Group* gr = NULL, bool force = false );
     // prefer isXXX() instead
-        NET::WindowType windowType( bool strict = false, int supported_types = SUPPORTED_WINDOW_TYPES_MASK ) const;
+        NET::WindowType windowType( bool direct = false, int supported_types = SUPPORTED_WINDOW_TYPES_MASK ) const;
+        const WindowRules* rules() const;
 
         QRect geometry() const;
         QSize size() const;
         QSize minSize() const;
+        QSize maxSize() const;
         QPoint pos() const;
         QRect rect() const;
         int x() const;
@@ -90,16 +96,16 @@ class Client : public QObject, public KDecorationDefines
             SizemodeAny,
             SizemodeFixedW, // try not to affect width
             SizemodeFixedH, // try not to affect height
-            SizemodeMax, // try not to make it larger in either direction
-            SizemodeShaded // shaded - height == 0
+            SizemodeMax // try not to make it larger in either direction
             };
         QSize adjustedSize( const QSize&, Sizemode mode = SizemodeAny ) const;
+        QSize adjustedSize() const;
 
         QPixmap icon() const;
         QPixmap miniIcon() const;
 
         bool isActive() const;
-        void setActive( bool );
+        void setActive( bool, bool updateOpacity = true );
 
         int desktop() const;
         void setDesktop( int );
@@ -111,13 +117,6 @@ class Client : public QObject, public KDecorationDefines
     // !isMinimized() && not hidden, i.e. normally visible on some virtual desktop
         bool isShown( bool shaded_is_shown ) const;
 
-        enum ShadeMode
-            {
-            ShadeNone, // not shaded
-            ShadeNormal, // normally shaded - isShade() is true only here
-            ShadeHover, // "shaded", but visible due to hover unshade
-            ShadeActivated // "shaded", but visible due to alt+tab to the window
-            };
         bool isShade() const; // true only for ShadeNormal
         ShadeMode shadeMode() const; // prefer isShade()
         void setShade( ShadeMode mode );
@@ -159,9 +158,6 @@ class Client : public QObject, public KDecorationDefines
         void setModal( bool modal );
         bool isModal() const;
 
-        bool storeSettings() const;
-        void setStoreSettings( bool );
-
     // auxiliary functions, depend on the windowType
         bool wantsTabFocus() const;
         bool wantsInput() const;
@@ -186,7 +182,8 @@ class Client : public QObject, public KDecorationDefines
         bool isResizable() const;
         bool isCloseable() const; // may be closed by the user (may have a close button)
 
-        void takeFocus( bool force, allowed_t );
+        void takeActivity( int flags, bool handled, allowed_t ); // takes ActivityFlags as arg (in utils.h)
+        void takeFocus( allowed_t );
         void demandAttention( bool set = true );
 
         void setMask( const QRegion& r, int mode = X::Unsorted );
@@ -206,7 +203,7 @@ class Client : public QObject, public KDecorationDefines
         // plainResize() simply resizes
         void plainResize( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void plainResize( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
-        void keepInArea( const QRect& area );
+        void keepInArea( QRect area, bool partial = false );
 
         void growHorizontal();
         void shrinkHorizontal();
@@ -214,19 +211,21 @@ class Client : public QObject, public KDecorationDefines
         void shrinkVertical();
 
         bool providesContextHelp() const;
+        KShortcut shortcut() const;
+        void setShortcut( const QString& cut );
 
-        bool performMouseCommand( Options::MouseCommand, QPoint globalPos );
+        bool performMouseCommand( Options::MouseCommand, QPoint globalPos, bool handled = false );
 
         QCString windowRole() const;
         QCString sessionId();
         QCString resourceName() const;
         QCString resourceClass() const;
         QCString wmCommand();
-        QCString wmClientMachine() const;
+        QCString wmClientMachine( bool use_localhost ) const;
         Window   wmClientLeader() const;
         pid_t pid() const;
 
-        QRect adjustedClientArea( const QRect& area ) const;
+        QRect adjustedClientArea( const QRect& desktop, const QRect& area ) const;
 
         Colormap colormap() const;
 
@@ -235,17 +234,18 @@ class Client : public QObject, public KDecorationDefines
     // updates visibility depending on whether it's on the current desktop
         void virtualDesktopChange();
 
-        QString caption() const;
+        QString caption( bool full = true ) const;
+        void updateCaption();
 
         void keyPressEvent( uint key_code ); // FRAME ??
-
+        void updateMouseGrab();
         Window moveResizeGrabWindow() const;
 
         const QPoint calculateGravitation( bool invert, int gravity = 0 ) const; // FRAME public?
 
         void NETMoveResize( int x_root, int y_root, NET::Direction direction );
         void NETMoveResizeWindow( int flags, int x, int y, int width, int height );
-        void restackWindow( Window above, int detail, NET::RequestSource source, bool send_event = false );
+        void restackWindow( Window above, int detail, NET::RequestSource source, Time timestamp, bool send_event = false );
         
         void gotPing( Time timestamp );
 
@@ -268,19 +268,37 @@ class Client : public QObject, public KDecorationDefines
         static bool belongToSameApplication( const Client* c1, const Client* c2, bool active_hack = false );
         static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon );
 
-    public slots: // FRAME these probably don't need to be slots anymore
-        void minimize();
-        void unminimize();
+        void minimize( bool avoid_animation = false );
+        void unminimize( bool avoid_animation = false );
         void closeWindow();
         void killWindow();
         void maximize( MaximizeMode );
-        void toggleOnAllDesktops();
         void toggleShade();
         void showContextHelp();
-        void autoRaise();
         void cancelAutoRaise();
-        void shadeHover();
         void destroyClient();
+        void checkActiveModal();
+        void setOpacity(bool translucent, uint opacity = 0);
+        void setShadowSize(uint shadowSize);
+        void updateOpacity();
+        void updateShadowSize();
+        bool hasCustomOpacity(){return custom_opacity;}
+        void setCustomOpacityFlag(bool custom = true);
+        bool getWindowOpacity();
+        int opacityPercentage();
+        void checkAndSetInitialRuledOpacity();
+        uint ruleOpacityInactive();
+        uint ruleOpacityActive();
+        unsigned int opacity();
+        bool isBMP();
+        void setBMP(bool b);
+        bool touches(const Client* c);
+        void setShapable(bool b);
+
+    private slots:
+        void autoRaise();
+        void shadeHover();
+        void shortcutActivated();
 
     private:
         friend class Bridge; // FRAME
@@ -332,22 +350,37 @@ class Client : public QObject, public KDecorationDefines
         bool isNormalState() const;
         bool isManaged() const; // returns false if this client is not yet managed
         void updateAllowedActions( bool force = false );
-        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny ) const;
+        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
         void changeMaximize( bool horizontal, bool vertical, bool adjust );
+        void checkMaximizeGeometry();
+        bool checkFullScreenHack( const QRect& geom ) const;
+        void updateFullScreenHack( const QRect& geom );
         void getWmNormalHints();
+        void getMotifHints();
         void getIcons();
         void getWmClientLeader();
+        void getWmClientMachine();
         void fetchName();
         void fetchIconicName();
+        QString readName() const;
+        void setCaption( const QString& s, bool force = false );
         bool hasTransientInternal( const Client* c, bool indirect, ConstClientList& set ) const;
+        void setupWindowRules( bool ignore_temporary );
+        void updateWindowRules();
+        void finishWindowRules();
+        void setShortcutInternal( const KShortcut& cut );
 
         void updateWorkareaDiffs();
         void checkDirection( int new_diff, int old_diff, QRect& rect, const QRect& area );
         static int computeWorkareaDiff( int left, int right, int a_left, int a_right );
-        void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity = 0 );
+        void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool );
         // resizeWithChecks() resizes according to gravity, and checks workarea position
         void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
+        NETExtendedStrut strut() const;
+        bool hasStrut() const;
+        int checkShadeGeometry( int w, int h );
+        void postponeGeometryUpdates( bool postpone );
 
         bool startMoveResize();
         void finishMoveResize( bool cancel );
@@ -355,15 +388,17 @@ class Client : public QObject, public KDecorationDefines
         void checkUnrestrictedMoveResize();
         void handleMoveResize( int x, int y, int x_root, int y_root );
         void positionGeometryTip();
-        void updateMouseGrab();
         void grabButton( int mod );
         void ungrabButton( int mod );
         void resetMaximize();
         void resizeDecoration( const QSize& s );
+        void setXTitleHeightProperty(int titleHeight);
 
         void pingWindow();
         void killProcess( bool ask, Time timestamp = CurrentTime );
         void updateUrgency();
+        static void sendClientMessage( Window w, Atom a, Atom protocol,
+            long data1 = 0, long data2 = 0, long data3 = 0 );
 
         void embedClient( Window w, const XWindowAttributes &attr );    
         void detectNoBorder();
@@ -373,8 +408,8 @@ class Client : public QObject, public KDecorationDefines
         void rawShow(); // just shows it
         void rawHide(); // just hides it
 
-        Time readUserTimeMapTimestamp( const KStartupInfoData* asn_data,
-            const SessionInfo* session ) const;
+        Time readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
+            bool session ) const;
         Time readUserCreationTime() const;
         static bool sameAppWindowRoleMatch( const Client* c1, const Client* c2, bool active_hack );
         void startupIdChanged();
@@ -428,11 +463,12 @@ class Client : public QObject, public KDecorationDefines
         uint original_skip_taskbar :1; // unaffected by KWin
         uint Pdeletewindow :1; // does the window understand the DeleteWindow protocol?
         uint Ptakefocus :1;// does the window understand the TakeFocus protocol?
+        uint Ptakeactivity : 1; // does it support _NET_WM_TAKE_ACTIVITY
         uint Pcontexthelp : 1; // does the window understand the ContextHelp protocol?
         uint Pping : 1; // does it support _NET_WM_PING?
         uint input :1; // does the window want input in its wm_hints
-        uint store_settings : 1;
         uint skip_pager : 1;
+        uint motif_noborder : 1;
         uint motif_may_resize : 1;
         uint motif_may_move :1;
         uint motif_may_close : 1;
@@ -445,6 +481,7 @@ class Client : public QObject, public KDecorationDefines
         uint not_obscured : 1;
         uint urgency : 1; // XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; // don't apply focus stealing prevention to this client
+        WindowRules client_rules;
         void getWMHints();
         void readIcons();
         void getWindowProtocols();
@@ -465,10 +502,10 @@ class Client : public QObject, public KDecorationDefines
         Colormap cmap;
         QCString resource_name;
         QCString resource_class;
+        QCString client_machine;
         QString cap_normal, cap_iconic, cap_suffix;
         WId wmClientLeaderWin;
         QCString window_role;
-        void checkGroup();
         Group* in_group;
         Window window_group;
         Layer in_layer;
@@ -479,16 +516,42 @@ class Client : public QObject, public KDecorationDefines
         unsigned long allowed_actions;
         QRect frame_geometry;
         QSize client_size;
-        int block_geometry; // >0 - new geometry is remembered, but not actually set
+        int postpone_geometry_updates; // >0 - new geometry is remembered, but not actually set
+        bool pending_geometry_update;
         bool shade_geometry_change;
         int border_left, border_right, border_top, border_bottom;
         QRegion _mask;
+        static bool check_active_modal; // see Client::checkActiveModal()
+        KShortcut _shortcut;
         friend struct FetchNameInternalPredicate;
         friend struct CheckIgnoreFocusStealingProcedure;
+        friend struct ResetupRulesProcedure;
+        friend class GeometryUpdatesPostponer;
         void show() { assert( false ); } // SELI remove after Client is no longer QWidget
         void hide() { assert( false ); }
+        uint opacity_;
+        uint savedOpacity_;
+        bool custom_opacity;
+        uint rule_opacity_active; //translucency rules
+        uint rule_opacity_inactive; //dto.
+        //int shadeOriginalHeight;
+        bool isBMP_;
+        
     };
 
+// helper for Client::postponeGeometryUpdates() being called in pairs (true/false)
+class GeometryUpdatesPostponer
+    {
+    public:
+        GeometryUpdatesPostponer( Client* c )
+            : cl( c ) { cl->postponeGeometryUpdates( true ); }
+        ~GeometryUpdatesPostponer()
+            { cl->postponeGeometryUpdates( false ); }
+    private:
+        Client* cl;
+    };
+
+
 // NET WM Protocol handler class
 class WinInfo : public NETWinInfo
     {
@@ -633,7 +696,7 @@ bool Client::isShade() const
     }
 
 inline
-Client::ShadeMode Client::shadeMode() const
+ShadeMode Client::shadeMode() const
     {
     return shade_mode;
     }
@@ -678,17 +741,6 @@ inline bool Client::keepBelow() const
     return keep_below;
     }
 
-inline bool Client::storeSettings() const
-    {
-    return store_settings;
-    }
-
-inline void Client::setStoreSettings( bool b )
-    {
-    store_settings = b;
-    }
-
-
 inline bool Client::shape() const
     {
     return is_shape;
@@ -755,11 +807,6 @@ inline QSize Client::size() const
     return frame_geometry.size();
     }
 
-inline QSize Client::minSize() const
-    {
-    return QSize( xSizeHint.min_width, xSizeHint.min_height );
-    }
-
 inline QPoint Client::pos() const
     {
     return frame_geometry.topLeft();
@@ -830,6 +877,11 @@ inline bool Client::ignoreFocusStealing() const
     return ignore_focus_stealing;
     }
 
+inline const WindowRules* Client::rules() const
+    {
+    return &client_rules;
+    }
+
 KWIN_PROCEDURE( CheckIgnoreFocusStealingProcedure, cl->ignore_focus_stealing = options->checkIgnoreFocusStealing( cl ));
 
 inline Window Client::moveResizeGrabWindow() const
@@ -837,10 +889,28 @@ inline Window Client::moveResizeGrabWindow() const
     return move_resize_grab_window;
     }
 
+inline KShortcut Client::shortcut() const
+    {
+    return _shortcut;
+    }
+
+inline bool Client::isBMP()
+    {
+    return isBMP_;
+    }
+
+inline void Client::setBMP(bool b)
+    {
+    isBMP_ = b;
+    }
+    
 #ifdef NDEBUG
-kndbgstream& operator<<( kndbgstream& stream, const Client* );
-kndbgstream& operator<<( kndbgstream& stream, const ClientList& );
-kndbgstream& operator<<( kndbgstream& stream, const ConstClientList& );
+inline
+kndbgstream& operator<<( kndbgstream& stream, const Client* ) { return stream; }
+inline
+kndbgstream& operator<<( kndbgstream& stream, const ClientList& ) { return stream; }
+inline
+kndbgstream& operator<<( kndbgstream& stream, const ConstClientList& ) { return stream; }
 #else
 kdbgstream& operator<<( kdbgstream& stream, const Client* );
 kdbgstream& operator<<( kdbgstream& stream, const ClientList& );
