@@ -182,6 +182,10 @@ void Workspace::updateClientArea( bool force )
              it != clients.end();
              ++it)
             (*it)->checkWorkspacePosition();
+        for( ClientList::ConstIterator it = desktops.begin();
+             it != desktops.end();
+             ++it)
+            (*it)->checkWorkspacePosition();
         }
     delete[] screens;
     delete[] new_sareas;
@@ -858,6 +862,13 @@ int Client::computeWorkareaDiff( int left, int right, int a_left, int a_right )
 
 void Client::checkWorkspacePosition()
     {
+    if( isDesktop())
+        {
+        QRect area = workspace()->clientArea( FullArea, this );
+        if( geometry() != area )
+            setGeometry( area );
+        return;
+        }
     if( maximizeMode() != MaximizeRestore )
 	// TODO update geom_restore?
         changeMaximize( false, false, true ); // adjust size
@@ -951,18 +962,6 @@ void Client::checkDirection( int new_diff, int old_diff, QRect& rect, const QRec
                 }
             return;
             }
-        if( isResizable())
-            {
-            if( rect.width() > area.width())
-                rect.setWidth( area.width());
-            if( rect.width() >= area.width() / 2 )
-                {
-                if( old_diff < 0 )
-                    rect.setLeft( area.left() + ( -old_diff - 1 ) );
-                else // old_diff > 0
-                    rect.setRight( area.right() - ( old_diff - 1 ));
-                }
-            }
         if( isMovable())
             {
             if( old_diff < 0 ) // was in left third, keep distance from left edge
@@ -970,16 +969,22 @@ void Client::checkDirection( int new_diff, int old_diff, QRect& rect, const QRec
             else // old_diff > 0 // was in right third, keep distance from right edge
                 rect.moveRight( area.right() - ( old_diff - 1 ));
             }
-        // this isResizable() block is copied from above, the difference is
-        // the condition with 'area.width() / 2' - for windows that are not wide,
-        // moving is preffered to resizing
-        if( isResizable())
+        else if( isResizable())
             {
             if( old_diff < 0 )
                 rect.setLeft( area.left() + ( -old_diff - 1 ) );
             else // old_diff > 0
                 rect.setRight( area.right() - ( old_diff - 1 ));
             }
+        if( rect.width() > area.width() && isResizable())
+            rect.setWidth( area.width());
+        if( isMovable())
+            {
+            if( rect.left() < area.left())
+                rect.moveLeft( area.left());
+            else if( rect.right() > area.right())
+                rect.moveRight( area.right());
+            }
         }
     if( rect.right() < area.left() + 5 || rect.left() > area.right() - 5 )
         { // not visible (almost) at all - try to make it at least partially visible
@@ -1006,6 +1011,13 @@ QSize Client::adjustedSize( const QSize& frame, Sizemode mode ) const
     return sizeForClientSize( wsize, mode, false );
     }
 
+// this helper returns proper size even if the window is shaded
+// see also the comment in Client::setGeometry()
+QSize Client::adjustedSize() const
+    {
+    return sizeForClientSize( clientSize());
+    }
+
 /*!
   Calculate the appropriate frame size for the given client size \a
   wsize.
@@ -1018,6 +1030,11 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
     {
     int w = wsize.width();
     int h = wsize.height();
+    if( w < 1 || h < 1 )
+        {
+        kdWarning() << "sizeForClientSize() with empty size!" << endl;
+        kdWarning() << kdBacktrace() << endl;
+        }
     if (w<1) w = 1;
     if (h<1) h = 1;
 
@@ -1072,6 +1089,9 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
         double min_aspect_h = xSizeHint.min_aspect.y; // and multiplying would go wrong otherwise
         double max_aspect_w = xSizeHint.max_aspect.x;
         double max_aspect_h = xSizeHint.max_aspect.y;
+        // According to ICCCM 4.1.2.3 PMinSize should be a fallback for PBaseSize for size increments,
+        // but not for aspect ratio. Since this code comes from FVWM, handles both at the same time,
+        // and I have no idea how it works, let's hope nobody relies on that.
         w -= xSizeHint.base_width;
         h -= xSizeHint.base_height;
         int max_width = max_size.width() - xSizeHint.base_width;
@@ -1121,6 +1141,8 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
         switch( mode )
             {
             case SizemodeAny:
+#if 0 // make SizemodeAny equal to SizemodeFixedW - prefer keeping fixed width,
+      // so that changing aspect ratio to a different value and back keeps the same size (#87298)
                 {
                 ASPECT_CHECK_SHRINK_H_GROW_W
                 ASPECT_CHECK_SHRINK_W_GROW_H
@@ -1128,6 +1150,7 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
                 ASPECT_CHECK_GROW_W
                 break;
                 }
+#endif
             case SizemodeFixedW:
                 {
                 // the checks are order so that attempts to modify height are first
@@ -1154,8 +1177,6 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
                 ASPECT_CHECK_GROW_H
                 break;
                 }
-            case SizemodeShaded:
-                break;
             }
 #undef ASPECT_CHECK_SHRINK_H_GROW_W
 #undef ASPECT_CHECK_SHRINK_W_GROW_H
@@ -1178,10 +1199,7 @@ QSize Client::sizeForClientSize( const QSize& wsize, Sizemode mode, bool noframe
         w += border_left + border_right;
         h += border_top + border_bottom;
         }
-    QSize ret = rules()->checkSize( QSize( w, h ));
-    if ( mode == SizemodeShaded && wsize.height() == 0 )
-        ret.setHeight( noframe ? 0 : border_top + border_bottom );
-    return ret;
+    return rules()->checkSize( QSize( w, h ));
     }
 
 /*!
@@ -1194,14 +1212,13 @@ void Client::getWmNormalHints()
         xSizeHint.flags = 0;
     // set defined values for the fields, even if they're not in flags
 
-    // basesize is just like minsize, except for minsize is not used for aspect ratios
-    // keep basesize only for aspect ratios, for size increments, keep the base
-    // value in minsize - see ICCCM 4.1.2.3
     if( xSizeHint.flags & PBaseSize )
         {
-        if( ! ( xSizeHint.flags & PMinSize )) // PBaseSize and PMinSize are equivalent
+        // PBaseSize is a fallback for PMinSize according to ICCCM 4.1.2.3
+        // The other way around PMinSize is not a complete fallback for PBaseSize,
+        // so that's not handled here.
+        if( ! ( xSizeHint.flags & PMinSize ))
             {
-            xSizeHint.flags |= PMinSize;
             xSizeHint.min_width = xSizeHint.base_width;
             xSizeHint.min_height = xSizeHint.base_height;
             }
@@ -1243,7 +1260,7 @@ void Client::getWmNormalHints()
         xSizeHint.win_gravity = NorthWestGravity;
     if( isManaged())
         { // update to match restrictions
-        QSize new_size = adjustedSize( size());
+        QSize new_size = adjustedSize();
         if( new_size != size() && !isFullScreen())
             resizeWithChecks( new_size );
         }
@@ -1385,7 +1402,7 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             || ns != size())
             {
             QRect orig_geometry = geometry();
-            ++block_geometry;
+            GeometryUpdatesPostponer blocker( this );
             move( new_pos );
             plainResize( ns );
             setGeometry( QRect( calculateGravitation( false, gravity ), size()));
@@ -1394,8 +1411,6 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen()
                 && area.contains( orig_geometry ))
                 keepInArea( area );
-            --block_geometry;
-            setGeometry( geometry(), ForceGeometrySet );
 
             // this is part of the kicker-xinerama-hack... it should be
             // safe to remove when kicker gets proper ExtendedStrut support;
@@ -1420,7 +1435,7 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
         if( ns != size())  // don't restore if some app sets its own size again
             {
             QRect orig_geometry = geometry();
-            ++block_geometry;
+            GeometryUpdatesPostponer blocker( this );
             int save_gravity = xSizeHint.win_gravity;
             xSizeHint.win_gravity = gravity;
             resizeWithChecks( ns );
@@ -1430,8 +1445,6 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
             if( !from_tool && ( !isSpecialWindow() || isToolbar()) && !isFullScreen()
                 && area.contains( orig_geometry ))
                 keepInArea( area );
-            --block_geometry;
-            setGeometry( geometry(), ForceGeometrySet );
             }
         }
     // No need to send synthetic configure notify event here, either it's sent together
@@ -1441,6 +1454,16 @@ void Client::configureRequest( int value_mask, int rx, int ry, int rw, int rh, i
 
 void Client::resizeWithChecks( int w, int h, ForceGeometry_t force )
     {
+    if( shade_geometry_change )
+        assert( false );
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdWarning() << "Shaded geometry passed for size:" << endl;
+            kdWarning() << kdBacktrace() << endl;
+            }
+        }
     int newx = x();
     int newy = y();
     QRect area = workspace()->clientArea( WorkArea, this );
@@ -1595,39 +1618,85 @@ bool Client::isMaximizable() const
  */
 void Client::setGeometry( int x, int y, int w, int h, ForceGeometry_t force )
     {
+    // this code is also duplicated in Client::plainResize()
+    // Ok, the shading geometry stuff. Generally, code doesn't care about shaded geometry,
+    // simply because there are too many places dealing with geometry. Those places
+    // ignore shaded state and use normal geometry, which they usually should get
+    // from adjustedSize(). Such geometry comes here, and if the window is shaded,
+    // the geometry is used only for client_size, since that one is not used when
+    // shading. Then the frame geometry is adjusted for the shaded geometry.
+    // This gets more complicated in the case the code does only something like
+    // setGeometry( geometry()) - geometry() will return the shaded frame geometry.
+    // Such code is wrong and should be changed to handle the case when the window is shaded,
+    // for example using Client::clientSize().
+    if( shade_geometry_change )
+        ; // nothing
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdDebug() << "Shaded geometry passed for size:" << endl;
+            kdDebug() << kdBacktrace() << endl;
+            }
+        else
+            {
+            client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+            h = border_top + border_bottom;
+            }
+        }
+    else
+        {
+        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+        }
     if( force == NormalGeometrySet && frame_geometry == QRect( x, y, w, h ))
         return;
-    h = checkShadeGeometry( w, h );
     frame_geometry = QRect( x, y, w, h );
-    if( !isShade())
-        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
-    else
-        client_size = QSize( w - border_left - border_right, client_size.height());
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
-        resizeDecoration( QSize( w, h ));
-        if( !isShade())
-            {
-            QSize cs = clientSize();
-            XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
-                cs.width(), cs.height());
-    	    // FRAME tady poradi tak, at neni flicker
-            XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
-            }
-        if( shape())
-            updateShape();
-        // SELI TODO won't this be too expensive?
-        updateWorkareaDiffs();
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
+    XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
+    resizeDecoration( QSize( w, h ));
+    if( !isShade())
+        {
+        QSize cs = clientSize();
+        XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
+            cs.width(), cs.height());
+        XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
+        }
+    if( shape())
+        updateShape();
+    // SELI TODO won't this be too expensive?
+    updateWorkareaDiffs();
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 void Client::plainResize( int w, int h, ForceGeometry_t force )
     {
+    // this code is also duplicated in Client::setGeometry(), and it's also commented there
+    if( shade_geometry_change )
+        ; // nothing
+    else if( isShade())
+        {
+        if( h == border_top + border_bottom )
+            {
+            kdDebug() << "Shaded geometry passed for size:" << endl;
+            kdDebug() << kdBacktrace() << endl;
+            }
+        else
+            {
+            client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+            h = border_top + border_bottom;
+            }
+        }
+    else
+        {
+        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
+        }
     if( QSize( w, h ) != rules()->checkSize( QSize( w, h )))
         {
         kdDebug() << "forced size fail:" << QSize( w,h ) << ":" << rules()->checkSize( QSize( w, h )) << endl;
@@ -1635,49 +1704,28 @@ void Client::plainResize( int w, int h, ForceGeometry_t force )
         }
     if( force == NormalGeometrySet && frame_geometry.size() == QSize( w, h ))
         return;
-    h = checkShadeGeometry( w, h );
     frame_geometry.setSize( QSize( w, h ));
-    if( !isShade())
-        client_size = QSize( w - border_left - border_right, h - border_top - border_bottom );
-    else
-        client_size = QSize( w - border_left - border_right, client_size.height());
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XResizeWindow( qt_xdisplay(), frameId(), w, h );
-        resizeDecoration( QSize( w, h ));
-        if( !isShade())
-            {
-            QSize cs = clientSize();
-            XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
-                cs.width(), cs.height());
-            XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
-            }
-        if( shape())
-            updateShape();
-        updateWorkareaDiffs();
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
-    }
-
-// There may be cases when an application requests resizing while shaded,
-// and even KWin itself may do so somewhere (too many places to check :-/ ).
-// If the requested geometry doesn't fit shaded geometry, adjust the height
-// of the requested geometry and return it.
-int Client::checkShadeGeometry( int w, int h )
-    {
-    // check that the frame is not resized to full size when it should be shaded
-    if( isShade() && !shade_geometry_change && h != border_top + border_bottom )
+    XResizeWindow( qt_xdisplay(), frameId(), w, h );
+    resizeDecoration( QSize( w, h ));
+    if( !isShade())
         {
-        kdDebug() << "Fixing shaded geometry:" << this << endl;
-        // adjust the client size to match the newly requested geometry
-        client_size = adjustedSize( QSize( w, h ));
-//        checkMaximizeGeometry(); // doesn't work, actual setting of geometry changes this again
-        h = border_top + border_bottom;
+        QSize cs = clientSize();
+        XMoveResizeWindow( qt_xdisplay(), wrapperId(), clientPos().x(), clientPos().y(),
+            cs.width(), cs.height());
+        XMoveResizeWindow( qt_xdisplay(), window(), 0, 0, cs.width(), cs.height());
         }
-    return h;
+    if( shape())
+        updateShape();
+    updateWorkareaDiffs();
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 /*!
@@ -1689,16 +1737,42 @@ void Client::move( int x, int y, ForceGeometry_t force )
         return;
     frame_geometry.moveTopLeft( QPoint( x, y ));
     updateWorkareaDiffs();
-    if( block_geometry == 0 )
+    if( postpone_geometry_updates != 0 )
         {
-        XMoveWindow( qt_xdisplay(), frameId(), x, y );
-        sendSyntheticConfigureNotify();
-        updateWindowRules();
-        checkMaximizeGeometry();
+        pending_geometry_update = true;
+        return;
         }
+    XMoveWindow( qt_xdisplay(), frameId(), x, y );
+    sendSyntheticConfigureNotify();
+    updateWindowRules();
+    checkMaximizeGeometry();
     }
 
 
+void Client::postponeGeometryUpdates( bool postpone )
+    {
+    if( postpone )
+        {
+        if( postpone_geometry_updates == 0 )
+            pending_geometry_update = false;
+        ++postpone_geometry_updates;
+        }
+    else
+        {
+        if( --postpone_geometry_updates == 0 )
+            {
+            if( pending_geometry_update )
+                {
+                if( isShade())
+                    setGeometry( QRect( pos(), adjustedSize()), ForceGeometrySet );
+                else
+                    setGeometry( geometry(), ForceGeometrySet );
+                pending_geometry_update = false;
+                }
+            }
+        }
+    }
+
 void Client::maximize( MaximizeMode m )
     {
     setMaximize( m & MaximizeVertical, m & MaximizeHorizontal );
@@ -1734,7 +1808,7 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     if( !adjust && max_mode == old_mode )
         return;
 
-    ++block_geometry; // TODO GeometryBlocker class?
+    GeometryUpdatesPostponer blocker( this );
 
     // maximing one way and unmaximizing the other way shouldn't happen
     Q_ASSERT( !( vertical && horizontal )
@@ -1855,9 +1929,6 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
             break;
         }
 
-    --block_geometry;
-    setGeometry( geometry(), ForceGeometrySet );
-
     updateAllowedActions();
     if( decoration != NULL )
         decoration->maximizeChange();
@@ -1874,7 +1945,10 @@ void Client::resetMaximize()
     updateAllowedActions();
     if( decoration != NULL )
         decoration->borders( border_left, border_right, border_top, border_bottom );
-    setGeometry( geometry(), ForceGeometrySet );
+    if( isShade())
+        setGeometry( QRect( pos(), sizeForClientSize( clientSize())), ForceGeometrySet );
+    else
+        setGeometry( geometry(), ForceGeometrySet );
     if( decoration != NULL )
         decoration->maximizeChange();
     }
@@ -1963,7 +2037,8 @@ void Client::setFullScreen( bool set, bool user )
     fullscreen_mode = set ? FullScreenNormal : FullScreenNone;
     if( was_fs == isFullScreen())
         return;
-    StackingUpdatesBlocker blocker( workspace());
+    StackingUpdatesBlocker blocker1( workspace());
+    GeometryUpdatesPostponer blocker2( this );
     workspace()->updateClientLayer( this ); // active fullscreens get different layer
     info->setState( isFullScreen() ? NET::FullScreen : 0, NET::FullScreen );
     updateDecoration( false, false );
