@@ -483,11 +483,11 @@ ClientList Workspace::constrainedStackingOrder()
         {
         Layer l = (*it)->layer();
         // If a window is raised above some other window in the same window group
-        // which is in a higher layer, make sure it stays above that window (see #95731).
-        if(( l == NormalLayer || l == AboveLayer )
-            && !(*it)->isDock()
-            && minimum_layer.contains( (*it)->group())
-            && l < minimum_layer[ (*it)->group() ] )
+        // which is in the ActiveLayer (i.e. it's fulscreened), make sure it stays
+        // above that window (see #95731).
+        if( minimum_layer.contains( (*it)->group())
+            && minimum_layer[ (*it)->group() ] == ActiveLayer
+            && ( l == NormalLayer || l == AboveLayer ))
             {
             l = minimum_layer[ (*it)->group() ];
             }
@@ -721,9 +721,10 @@ Layer Client::belongsToLayer() const
     // only raise fullscreen above docks if it's the topmost window in unconstrained stacking order,
     // i.e. the window set to be topmost by the user (also includes transients of the fullscreen window)
     const Client* ac = workspace()->mostRecentlyActivatedClient(); // instead of activeClient() - avoids flicker
-    if( isFullScreen() && ac != NULL
-        && workspace()->topClientOnDesktop( desktop(), true ) == ac
-        && ( ac == this || this->hasTransient( ac, true )))
+    const Client* top = workspace()->topClientOnDesktop( desktop(), true );
+    if( isFullScreen() && ac != NULL && top != NULL
+          && ( ac == this || this->group() == ac->group())
+          && ( top == this || this->group() == top->group()))
         return ActiveLayer;
     if( keepAbove())
         return AboveLayer;
